@@ -38,8 +38,7 @@ class AstroImage(BaseImage):
         cls.ioClass = klass
 
     def __init__(self, data_np=None, metadata=None, logger=None,
-                 name=None, wcsclass=None, ioclass=None,
-                 inherit_primary_header=False, save_primary_header=True):
+                 name=None, wcsclass=None, ioclass=None, **kwargs):
 
         BaseImage.__init__(self, data_np=data_np, metadata=metadata,
                            logger=logger, name=name)
@@ -62,13 +61,22 @@ class AstroImage(BaseImage):
 
         self.io = ioclass(self.logger)
 
+        # these attributes to be removed--DO NOT USE!!!
+        inherit_primary_header = False
+        if 'inherit_primary_header' in kwargs:
+            warnings.warn("inherit_primary_header kwarg has been deprecated--"
+                          "use inherit_primary_header in load_hdu() or load_file()",
+                          DeprecationWarning)
+            inherit_primary_header = kwargs['inherit_primary_header']
         self.inherit_primary_header = inherit_primary_header
-        self.save_primary_header = inherit_primary_header or save_primary_header
-        if self.save_primary_header:
-            # User wants to save/inherit from primary header--this will hold it
-            self._primary_hdr = AstroHeader()
-        else:
-            self._primary_hdr = None
+
+        save_primary_header = True
+        if 'save_primary_header' in kwargs:
+            warnings.warn("save_primary_header kwarg has been deprecated--"
+                          "use save_primary_header in load_hdu() or load_file()",
+                          DeprecationWarning)
+            save_primary_header = kwargs['save_primary_header']
+        self.save_primary_header = save_primary_header
 
         if metadata is not None:
             header = self.get_header()
@@ -106,41 +114,23 @@ class AstroImage(BaseImage):
         self.set_naxispath(naxispath)
 
     def load_hdu(self, hdu, fobj=None, naxispath=None,
-                 inherit_primary_header=None):
+                 save_primary_header=None, inherit_primary_header=None,
+                 **kwargs):
+        """NOTE: this is for astropy.io.fits HDUs only."""
 
-        # this seems to be necessary now for some fits files...
-        try:
-            hdu.verify('fix')
+        if self.io is None:
+            raise ImageError("No IO loader defined")
 
-        except Exception as e:
-            # Let's hope for the best!
-            self.logger.warning("Problem verifying fits HDU: {}".format(e))
-
-        self.clear_metadata()
-
-        # collect HDU header
-        ahdr = self.get_header()
-        self._copy_hdu_header(hdu, ahdr)
-
-        # Set PRIMARY header
+        # TO BE DEPRECATED--will just pass kwargs on through
+        if save_primary_header is None:
+            save_primary_header = self.save_primary_header
         if inherit_primary_header is None:
             inherit_primary_header = self.inherit_primary_header
-        else:  # This ensures get_header() is consistent
-            self.inherit_primary_header = inherit_primary_header
 
-        save_primary_header = (self.save_primary_header or
-                               inherit_primary_header)
-        if save_primary_header and (fobj is not None):
-            if self._primary_hdr is None:
-                self._primary_hdr = AstroHeader()
-
-            self._copy_hdu_header(fobj[0], self._primary_hdr)
-
-        self.setup_data(hdu.data, naxispath=naxispath)
-
-        # Try to make a wcs object on the header
-        if hasattr(self, 'wcs') and self.wcs is not None:
-            self.wcs.load_header(hdu.header, fobj=fobj)
+        self.io.load_hdu(hdu, dstobj=self, fobj=fobj, naxispath=naxispath,
+                         save_primary_header=save_primary_header,
+                         inherit_primary_header=inherit_primary_header,
+                         **kwargs)
 
     def load_nddata(self, ndd, naxispath=None):
         """Load from an astropy.nddata.NDData object.
@@ -169,6 +159,12 @@ class AstroImage(BaseImage):
         if self.io is None:
             raise ImageError("No IO loader defined")
 
+        # TO BE DEPRECATED--will just pass kwargs on through
+        if 'save_primary_header' not in kwargs:
+            kwargs['save_primary_header'] = self.save_primary_header
+        if 'inherit_primary_header' not in kwargs:
+            kwargs['inherit_primary_header'] = self.inherit_primary_header
+
         self.io.load_file(filespec, dstobj=self, **kwargs)
 
     def load_data(self, data_np, naxispath=None, metadata=None):
@@ -182,7 +178,7 @@ class AstroImage(BaseImage):
 
     def load_buffer(self, buf, dims, dtype, byteswap=False,
                     naxispath=None, metadata=None):
-        data = np.fromstring(buf, dtype=dtype)
+        data = np.frombuffer(buf, dtype=dtype)
         if byteswap:
             data.byteswap(True)
         data = data.reshape(dims)
@@ -222,35 +218,33 @@ class AstroImage(BaseImage):
         return self.get_size()
 
     def get_header(self, create=True, include_primary_header=None):
-        try:
-            # By convention, the fits header is stored in a dictionary
-            # under the metadata keyword 'header'
-            hdr = self.metadata['header']
-
-            if include_primary_header is None:
-                include_primary_header = self.inherit_primary_header
-
-            if include_primary_header and self._primary_hdr is not None:
-                # Inherit PRIMARY header for display but keep metadata intact
-                displayhdr = AstroHeader()
-                for key in hdr.keyorder:
-                    card = hdr.get_card(key)
-                    bnch = displayhdr.__setitem__(card.key, card.value)
-                    bnch.comment = card.comment
-                for key in self._primary_hdr.keyorder:
-                    if key not in hdr:
-                        card = self._primary_hdr.get_card(key)
-                        bnch = displayhdr.__setitem__(card.key, card.value)
-                        bnch.comment = card.comment
-            else:
-                # Normal, separate header
-                displayhdr = hdr
-
-        except KeyError as e:
+        # By convention, the fits header is stored in a dictionary
+        # under the metadata keyword 'header'
+        if 'header' not in self:
             if not create:
-                raise e
+                # TODO: change to ValueError("No header found")
+                raise KeyError('header')
+
             hdr = AstroHeader()
-            self.metadata['header'] = hdr
+            self.set(header=hdr)
+        else:
+            hdr = self['header']
+
+        if include_primary_header is None:
+            include_primary_header = (self.get('inherit_primary_header', False) or
+                                      # TO BE DEPRECATED
+                                      self.inherit_primary_header)
+
+        # If it was saved during loading, by convention, the primary fits
+        # header is stored in a dictionary under the keyword 'primary_header'
+        if include_primary_header and self.has_primary_header():
+            # Inherit PRIMARY header for display but keep metadata intact
+            displayhdr = AstroHeader()
+            displayhdr.merge(hdr)
+            primary_hdr = self['primary_header']
+            displayhdr.merge(primary_hdr, override_keywords=False)
+        else:
+            # Normal, separate header
             displayhdr = hdr
 
         return displayhdr
@@ -276,10 +270,10 @@ class AstroImage(BaseImage):
             prev = kwds[kwd]  # noqa, this raises KeyError
         kwds[kwd] = value
 
-    def update_keywords(self, keyDict):
+    def update_keywords(self, key_dict):
         hdr = self.get_header()
         # Upcase all keywords
-        for kwd, val in keyDict.items():
+        for kwd, val in key_dict.items():
             hdr[kwd.upper()] = val
 
         # Try to make a wcs object on the header
@@ -297,8 +291,7 @@ class AstroImage(BaseImage):
                       astype=astype)
 
     def update_metadata(self, key_dict):
-        for key, val in key_dict.items():
-            self.metadata[key] = val
+        self.update(key_dict)
 
         # refresh the WCS
         if hasattr(self, 'wcs') and self.wcs is not None:
@@ -306,7 +299,8 @@ class AstroImage(BaseImage):
             self.wcs.load_header(header)
 
     def has_primary_header(self):
-        return self._primary_hdr is not None
+        primary_hdr = self.get('primary_header', None)
+        return isinstance(primary_hdr, AstroHeader)
 
     def clear_all(self):
         # clear metadata and data
@@ -357,16 +351,6 @@ class AstroImage(BaseImage):
         hdu = fits.PrimaryHDU(data=data, header=header)
         return hdu
 
-    def _copy_hdu_header(self, hdu, ahdr):
-        """Copy a FITS header from an astropy.io.fits.PrimaryHDU object
-        into a ginga.AstroImage.AstroHeader object.
-        """
-        header = hdu.header
-        for card in header.cards:
-            if len(card.keyword) == 0:
-                continue
-            ahdr.set_card(card.keyword, card.value, comment=card.comment)
-
     def astype(self, type_name):
         """Convert AstroImage object to some other kind of object.
         """
@@ -414,57 +398,57 @@ class AstroImage(BaseImage):
     def get_starsep_XY(self, x1, y1, x2, y2):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.get_starsep_XY(self, x1, y1, x2, y2)
 
     def calc_radius_xy(self, x, y, radius_deg):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_radius_xy(self, x, y, radius_deg)
 
     def calc_radius_deg2pix(self, ra_deg, dec_deg, delta_deg,
                             equinox=None):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_radius_deg2pix(self, ra_deg, dec_deg, delta_deg,
                                        equinox=equinox)
 
     def add_offset_xy(self, x, y, delta_deg_x, delta_deg_y):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.add_offset_xy(self, x, y, delta_deg_x, delta_deg_y)
 
     def calc_radius_center(self, delta_deg):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_radius_center(self, delta_deg)
 
     def calc_compass(self, x, y, len_deg_e, len_deg_n):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_compass(self, x, y, len_deg_e, len_deg_n)
 
     def calc_compass_radius(self, x, y, radius_px):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_compass_radius(self, x, y, radius_px)
 
     def calc_compass_center(self):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         return wcs.calc_compass_center(self)
 
     def get_wcs_rotation_deg(self):
         warnings.warn("This function has been deprecated--"
                       "use get_rotation_and_scale in ginga.util.wcs",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         header = self.get_header()
         (rot, cdelt1, cdelt2) = wcs.get_rotation_and_scale(header)
         return rot
@@ -475,7 +459,7 @@ class AstroImage(BaseImage):
                       update_minmax=True, suppress_callback=False):
         warnings.warn("This function has been deprecated--"
                       "use the version in ginga.util.mosaic",
-                      PendingDeprecationWarning)
+                      DeprecationWarning)
         from ginga.util import mosaic
         return mosaic.mosaic_inline(self, imagelist, bg_ref=bg_ref,
                                     trim_px=trim_px, merge=merge,
@@ -521,17 +505,24 @@ class AstroImage(BaseImage):
                     if system in ('galactic', 'ecliptic'):
                         sign, deg, min, sec = wcs.degToDms(lon_deg,
                                                            isLatitude=False)
-                        ra_txt = '+%03d:%02d:%06.3f' % (deg, min, sec)
+                        ra_fmt = '+%03d:%02d:%02d.%03d'
                     else:
                         deg, min, sec = wcs.degToHms(lon_deg)
-                        ra_txt = '%02d:%02d:%06.3f' % (deg, min, sec)
+                        ra_fmt = '%02d:%02d:%02d.%03d'
+
+                    frac_sec, sec = np.modf(sec)
+                    frac_sec = int(frac_sec * 1000)
+                    ra_txt = ra_fmt % (deg, min, sec, frac_sec)
 
                     sign, deg, min, sec = wcs.degToDms(lat_deg)
                     if sign < 0:
                         sign = '-'
                     else:
                         sign = '+'
-                    dec_txt = '%s%02d:%02d:%06.3f' % (sign, deg, min, sec)
+                    frac_sec, sec = np.modf(sec)
+                    frac_sec = int(frac_sec * 1000)
+                    dec_txt = '%s%02d:%02d:%02d.%03d' % (sign, deg, min, sec,
+                                                         frac_sec)
 
                 else:
                     ra_txt = '%+10.7f' % (lon_deg)
@@ -562,5 +553,3 @@ class AstroImage(BaseImage):
         info.update(dict(itype='astro', ra_txt=ra_txt, dec_txt=dec_txt,
                          ra_lbl=ra_lbl, dec_lbl=dec_lbl))
         return info
-
-# END
