@@ -68,6 +68,9 @@ class ViewerObjectBase(Callback.Callbacks):
     def __getitem__(self, kwd):
         return self.metadata[kwd]
 
+    def __contains__(self, kwd):
+        return kwd in self.metadata
+
     def update(self, kwds):
         self.metadata.update(kwds)
 
@@ -241,20 +244,7 @@ class BaseImage(ViewerObjectBase):
         if order is not None and order != '':
             self.order = order.upper()
         else:
-            shape = self.shape
-            if len(shape) <= 2:
-                self.order = 'M'
-            else:
-                depth = shape[-1]
-                # TODO: need something better here than a guess!
-                if depth == 1:
-                    self.order = 'M'
-                elif depth == 2:
-                    self.order = 'MA'
-                elif depth == 3:
-                    self.order = 'RGB'
-                elif depth == 4:
-                    self.order = 'RGBA'
+            self.order = trcalc.guess_order(self.shape)
 
     def has_valid_wcs(self):
         return hasattr(self, 'wcs') and self.wcs.has_valid_wcs()
@@ -293,7 +283,15 @@ class BaseImage(ViewerObjectBase):
 
     # kwargs is needed so subclasses can interoperate with optional keywords.
     def get_header(self, **kwargs):
-        return self.get('header', Header())
+        header = self.get('header', None)
+        if header is None:
+            header = Header()
+            self.set(header=header)
+        return header
+
+    def update_keywords(self, key_dict):
+        hdr = self.get_header()
+        hdr.update(key_dict)
 
     def transfer(self, other, astype=None):
         data = self._get_data()
@@ -391,9 +389,8 @@ class BaseImage(ViewerObjectBase):
         Return full mask where True marks pixels within the given shape.
         """
         wd, ht = self.get_size()
-        yi = np.mgrid[:ht].reshape(-1, 1)
-        xi = np.mgrid[:wd].reshape(1, -1)
-        pts = np.asarray((xi, yi)).T
+        xi, yi = np.meshgrid(range(0, wd), range(0, ht))
+        pts = np.array((xi, yi)).T
         contains = shape_obj.contains_pts(pts)
         return contains
 
@@ -415,9 +412,8 @@ class BaseImage(ViewerObjectBase):
             y1, y2 = max(0, y1), min(y2, ht - 1)
 
         # calculate pixel containment mask in bbox
-        yi = np.mgrid[y1:y2 + 1].reshape(-1, 1)
-        xi = np.mgrid[x1:x2 + 1].reshape(1, -1)
-        pts = np.asarray((xi, yi)).T
+        xi, yi = np.meshgrid(range(x1, x2 + 1), range(y1, y2 + 1))
+        pts = np.array((xi, yi)).T
         contains = shape_obj.contains_pts(pts)
 
         view = np.s_[y1:y2 + 1, x1:x2 + 1]
@@ -629,6 +625,14 @@ class Header(dict):
         except KeyError:
             return alt
 
+    def merge(self, hdr, override_keywords=False):
+        if not isinstance(hdr, Header):
+            raise ValueError("need to pass a compatible header for merge")
+        for key in hdr.keys():
+            if key not in self or override_keywords:
+                card = hdr.get_card(key)
+                self.set_card(key, card.value, comment=card.comment)
+
     def update(self, map_kind):
         for key, value in map_kind.items():
             self.__setitem__(key, value)
