@@ -98,7 +98,7 @@ class LinearDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         val = np.clip(pct, 0.0, 1.0)
         return val
 
@@ -127,7 +127,7 @@ class LogDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         val_inv = (np.exp(pct * np.log(self.exp)) - 1) / self.exp
         val = np.clip(val_inv, 0.0, 1.0)
         return val
@@ -157,7 +157,7 @@ class PowerDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         val_inv = np.log(self.exp * pct + 1) / np.log(self.exp)
         val = np.clip(val_inv, 0.0, 1.0)
         return val
@@ -186,7 +186,7 @@ class SqrtDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         val_inv = pct ** 2.0
         val = np.clip(val_inv, 0.0, 1.0)
         return val
@@ -214,7 +214,7 @@ class SquaredDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         val_inv = np.sqrt(pct)
         val = np.clip(val_inv, 0.0, 1.0)
         return val
@@ -246,7 +246,7 @@ class AsinhDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         # calculate inverse of dist fn
         val_inv = np.sinh(self.nonlinearity * pct) / self.factor
         val = np.clip(val_inv, 0.0, 1.0)
@@ -279,7 +279,7 @@ class SinhDist(ColorDistBase):
         self.check_hash()
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         # calculate inverse of dist fn
         val_inv = np.arcsinh(self.nonlinearity * pct) / self.factor
         val = np.clip(val_inv, 0.0, 1.0)
@@ -328,7 +328,7 @@ class HistogramEqualizationDist(ColorDistBase):
         return arr
 
     def get_dist_pct(self, pct):
-        pct = np.asarray(pct, dtype=np.float)
+        pct = np.asarray(pct, dtype=float)
         # TODO: this is wrong but we need a way to invert the hash
         val = np.clip(pct, 0.0, 1.0)
         return pct
@@ -337,6 +337,32 @@ class HistogramEqualizationDist(ColorDistBase):
         return 'histeq'
 
 
+class CurveDist(ColorDistBase):
+    """
+    y = x
+        where x in (0..1)
+    """
+
+    def __init__(self, hashsize, colorlen=None):
+        super(CurveDist, self).__init__(hashsize, colorlen=colorlen)
+
+    def calc_hash(self):
+        base = np.arange(0.0, float(self.hashsize), 1.0) / self.hashsize
+        # normalize to color range
+        l = base * (self.colorlen - 1)
+        self.hash = l.astype(np.uint, copy=False)
+
+        self.check_hash()
+
+    def get_dist_pct(self, pct):
+        pct = np.asarray(pct, dtype=float)
+        val = np.clip(pct, 0.0, 1.0)
+        return val
+
+    def __str__(self):
+        return 'curve'
+
+
 distributions = {
     'linear': LinearDist,
     'log': LogDist,
