@@ -163,10 +163,10 @@ class RenderContext(render.RenderContextBase):
         self.cr.canvas.path(path, self.pen, self.brush)
 
 
-class CanvasRenderer(render.StandardPixelRenderer):
+class CanvasRenderer(render.StandardPipelineRenderer):
 
     def __init__(self, viewer):
-        render.StandardPixelRenderer.__init__(self, viewer)
+        render.StandardPipelineRenderer.__init__(self, viewer)
 
         self.kind = 'agg'
         self.rgb_order = 'RGBA'
@@ -185,8 +185,8 @@ class CanvasRenderer(render.StandardPixelRenderer):
 
         super(CanvasRenderer, self).resize(dims)
 
-    def render_image(self, rgbobj, dst_x, dst_y):
-        """Render the image represented by (rgbobj) at dst_x, dst_y
+    def render_image(self, data, order, win_coord):
+        """Render the image represented by (data) at (win_coord)
         in the pixel space.
         *** internal method-- do not use ***
         """
@@ -195,12 +195,13 @@ class CanvasRenderer(render.StandardPixelRenderer):
         self.logger.debug("redraw surface")
 
         # get window contents as a buffer and load it into the AGG surface
-        rgb_buf = self.viewer.getwin_buffer(order=self.rgb_order,
-                                            dtype=np.uint8)
+        rgb_buf = data.tobytes(order='C')
         self.surface.frombytes(rgb_buf)
 
         # for debugging
-        #self.save_rgb_image_as_file('/tmp/temp.png', format='png')
+        # import os.path, tempfile
+        # self.save_rgb_image_as_file(os.path.join(tempfile.gettempdir(),
+        #                                          'agg_out.png', format='png'))
 
     def get_surface_as_array(self, order=None):
         if self.surface is None:
@@ -211,7 +212,7 @@ class CanvasRenderer(render.StandardPixelRenderer):
         wd, ht = self.dims
 
         # Get agg surface as a numpy array
-        arr8 = np.fromstring(self.surface.tobytes(), dtype=np.uint8)
+        arr8 = np.frombuffer(self.surface.tobytes(), dtype=np.uint8)
         arr8 = arr8.reshape((ht, wd, len(self.rgb_order)))
 
         # adjust according to viewer's needed order
