@@ -6,6 +6,7 @@
 #
 import numpy as np
 from collections import namedtuple
+import copy
 
 from ginga.misc import Callback, Bunch
 from ginga import trcalc, colors
@@ -126,6 +127,13 @@ class CanvasObjectBase(Callback.Callbacks):
     def is_compound(self):
         return False
 
+    def copy(self, share=[]):
+        obj = copy.copy(self)
+        obj.viewer = None
+        if 'data' not in share:
+            obj.data = None
+        return obj
+
     def contains_pts(self, points):
         contains = np.asarray([False] * len(points))
         return contains
@@ -283,7 +291,7 @@ class CanvasObjectBase(Callback.Callbacks):
     def set_point_by_index(self, i, pt):
         #self.points[i] = self.crdmap.data_to(pt)
         # Can we eventually use something like the above?
-        points = np.asarray(self.points, dtype=np.float)
+        points = np.asarray(self.points, dtype=float)
         points[i] = self.crdmap.data_to(pt)
         self.points = points
 
@@ -519,6 +527,10 @@ class CanvasObjectBase(Callback.Callbacks):
         else:
             return trcalc.strip_z(trcalc.get_bounds(points))
 
+    def get_llur(self):
+        a, b = trcalc.get_bounds(self.get_data_points())
+        return (a[0], a[1], b[0], b[1])
+
     # --- TO BE DEPRECATED METHODS ---
 
     def move_delta(self, xoff, yoff):
