@@ -5,8 +5,6 @@
 # Please see the file LICENSE.txt for details.
 
 from io import BytesIO
-import math
-import time
 
 import numpy as np
 
@@ -20,7 +18,9 @@ except ImportError:
 
 from ginga import trcalc, RGBMap
 from ginga.fonts import font_asst
-from ginga.util import rgb_cms
+from ginga.util import pipeline, rgb_cms
+from ginga.util.stages import render
+from ginga.misc import Bunch
 
 
 class RenderError(Exception):
@@ -40,6 +40,9 @@ class RenderContextBase(object):
         # TO BE EVENTUALLY DEPRECATED
         return self.renderer.scale_fontsize(fontsize)
 
+    def draw_image(self, cvs_img, cpoints, cache, whence, order='RGBA'):
+        pass
+
 
 class RendererBase(object):
     """Base class from which all Renderer classes are derived."""
@@ -53,10 +56,67 @@ class RendererBase(object):
         #raise RenderError("subclass should override this method!")
         pass
 
+    def render_whence(self, whence):
+        """Called in eventual response to a redraw request in the viewer.
+
+        Causes the rendering pipeline to be partially or completely
+        re-executed, depending on `whence`.
+
+        Parameters
+        ----------
+        whence : `float` or `int`
+            A number indicating at what level things need to be redrawn.
+
+        """
+        raise RenderError("subclass should override this method!")
+
     def finalize(self):
         #raise RenderError("subclass should override this method!")
         pass
 
+    def prepare_image(self, cvs_img, cache, whence):
+        """Prepare an image to be displayed somewhere in the viewer.
+
+        Called at a point in the rendering process when it is known
+        that an image will need to be displayed.  In this renderer,
+        it is called via the "overlays" stage, and it creates an RGB
+        image in the cache which is then merged into the background.
+
+        Parameters
+        ----------
+        cvs_img : instance of `~ginga.canvas.types.image.Image`
+            The canvas object for the image to be displayed
+
+        cache : `~ginga.misc.Bunch.Bunch`
+            A cache of items used by the renderer for a particular viewer
+
+        whence : `float` or `int`
+            A number indicating at what level things need to be redrawn.
+        """
+        raise RenderError("subclass should override this method!")
+
+    def getwin_array(self, order='RGBA', alpha=1.0, dtype=None):
+        """Get Numpy data array representing the viewer window.
+
+        Parameters
+        ----------
+        order : str
+            The desired order of RGB color layers.
+
+        alpha : float
+            Opacity.
+
+        dtype : numpy dtype
+            Numpy data type desired; defaults to the renderer default.
+
+        Returns
+        -------
+        outarr : ndarray
+            Numpy data array for display window.
+
+        """
+        raise RenderError("subclass should override this method!")
+
     def get_surface_as_array(self, order=None):
         raise RenderError("subclass should override this method!")
 
@@ -163,6 +223,72 @@ class RendererBase(object):
     def limits_change(self, limits):
         self.viewer.redraw(whence=3)
 
+    def get_scale(self):
+        """Return the scale currently set in the renderer."""
+        raise RenderError("subclass should override this method!")
+
+    def get_origin(self):
+        """Return the point in data coordinates that is supposed to
+        be at the pan position (center of the window)."""
+        raise RenderError("subclass should override this method!")
+
+    def get_window_center(self):
+        """Return the point in pixel coordinates at the center
+        of the window."""
+        raise RenderError("subclass should override this method!")
+
+    def get_window_size(self):
+        """Return the window dimensions."""
+        raise RenderError("subclass should override this method!")
+
+    def resize(self, dims):
+        raise RenderError("subclass should override this method!")
+
+    def calc_const_len(self, clen):
+        # For standard pixel renderer, pixel size is constant
+        raise RenderError("subclass should override this method!")
+
+    def _confirm_pan_and_scale(self, scale_x, scale_y, pan_x, pan_y,
+                               win_wd, win_ht):
+        raise RenderError("subclass should override this method!")
+
+    def scale_fontsize(self, fontsize):
+        raise RenderError("subclass should override this method!")
+
+
+def get_render_class(rtype):
+
+    rtype = rtype.lower()
+    if rtype == 'pil':
+        from ginga.pilw import CanvasRenderPil
+        return CanvasRenderPil.CanvasRenderer
+
+    if rtype == 'agg':
+        from ginga.aggw import CanvasRenderAgg
+        return CanvasRenderAgg.CanvasRenderer
+
+    if rtype == 'opencv':
+        from ginga.cvw import CanvasRenderCv
+        return CanvasRenderCv.CanvasRenderer
+
+    if rtype == 'cairo':
+        from ginga.cairow import CanvasRenderCairo
+        return CanvasRenderCairo.CanvasRenderer
+
+    if rtype == 'opengl':
+        from ginga.opengl import CanvasRenderGL
+        return CanvasRenderGL.CanvasRenderer
+
+    if rtype == 'qt':
+        from ginga.qtw import CanvasRenderQt
+        return CanvasRenderQt.CanvasRenderer
+
+    if rtype == 'vqt':
+        from ginga.qtw import CanvasRenderQt
+        return CanvasRenderQt.VectorCanvasRenderer
+
+    raise ValueError("Don't know about '%s' renderer type" % (rtype))
+
 
 class StandardPixelRenderer(RendererBase):
     """Standard renderer for generating bitmap-based image that can be
@@ -275,20 +401,15 @@ class StandardPixelRenderer(RendererBase):
         if self._rgbarr is None:
             self.create_bg_array(win_wd, win_ht, order)
 
-        t1 = t2 = t3 = time.time()
-
         if (whence <= 2.0) or (self._rgbarr2 is None):
             # Apply any RGB image overlays
             self._rgbarr2 = np.copy(self._rgbarr)
             p_canvas = self.viewer.get_private_canvas()
             self._overlay_images(p_canvas, whence=whence)
 
-            t2 = time.time()
-
         output_profile = self.viewer.t_.get('icc_output_profile', None)
         if output_profile is None:
             self._rgbarr3 = self._rgbarr2
-            t3 = t2
 
         elif (whence <= 2.3) or (self._rgbarr3 is None):
             self._rgbarr3 = np.copy(self._rgbarr2)
@@ -299,8 +420,6 @@ class StandardPixelRenderer(RendererBase):
                 self.convert_via_profile(self._rgbarr3, order,
                                          working_profile, output_profile)
 
-            t3 = time.time()
-
         if (whence <= 2.5) or (self._rgbarr4 is None):
             data = np.copy(self._rgbarr3)
 
@@ -317,15 +436,18 @@ class StandardPixelRenderer(RendererBase):
 
             self._rgbobj = RGBMap.RGBPlanes(rotimg, order)
 
-        t4 = time.time()
-        self.logger.debug("times: t2=%.4f t3=%.4f t4=%.4f total=%.4f" % (
-            t2 - t1, t3 - t2, t4 - t3, t4 - t1))
-
         return self._rgbobj
 
     def render_whence(self, whence):
         rgbobj = self.get_rgb_object(whence=whence)
-        self.render_image(rgbobj, self._dst_x, self._dst_y)
+
+        dst_order = self.rgb_order
+        #rgbarr = rgbobj.get_array(dst_order, dtype=np.uint8)
+        #self.render_image(rgbarr, dst_order, (self._dst_x, self._dst_y))
+
+        rgbarr = self.getwin_array(order=dst_order, alpha=1.0,
+                                   dtype=np.uint8)
+        self.render_image(rgbarr, dst_order, (0, 0))
 
     def _calc_bg_dimensions(self, win_wd, win_ht):
         """Calculate background image size necessary for rendering.
@@ -343,7 +465,7 @@ class StandardPixelRenderer(RendererBase):
 
         # Make a square from the scaled cutout, with room to rotate
         slop = 20
-        side = int(math.sqrt(win_wd**2 + win_ht**2) + slop)
+        side = int(np.sqrt(win_wd**2 + win_ht**2) + slop)
         wd = ht = side
 
         # Find center of new array
@@ -375,8 +497,8 @@ class StandardPixelRenderer(RendererBase):
         sx = float(win_wd) / scale_x
         sy = float(win_ht) / scale_y
         if (sx < 1.0) or (sy < 1.0):
-            #self.logger.warning("new scale would exceed max/min; scale unchanged")
-            raise RenderError("new scale would exceed pixel max; scale unchanged")
+            if self.viewer.settings.get('sanity_check_scale', True):
+                raise RenderError("new scale would exceed pixel max; scale unchanged")
 
         # record location of pan position pixel
         self._org_x, self._org_y = pan_x - data_off, pan_y - data_off
@@ -579,8 +701,6 @@ class StandardPixelRenderer(RendererBase):
             return
         dstarr = self._rgbarr2
 
-        t1 = t2 = time.time()
-
         self._common_draw(cvs_img, cache, whence)
 
         if cache.cutout is None:
@@ -588,7 +708,6 @@ class StandardPixelRenderer(RendererBase):
 
         cache.rgbarr = cache.cutout
 
-        t2 = time.time()
         # should this be self.get_rgb_order() ?
         dst_order = self.viewer.get_rgb_order()
         image_order = cvs_img.image.get_order()
@@ -600,23 +719,17 @@ class StandardPixelRenderer(RendererBase):
                              alpha=cvs_img.alpha, fill=True, flipy=False)
 
         cache.drawn = True
-        t3 = time.time()
-        self.logger.debug("draw: t2=%.4f t3=%.4f total=%.4f" % (
-            t2 - t1, t3 - t2, t3 - t1))
 
     def _prepare_norm_image(self, cvs_img, cache, whence):
         if whence > 2.3 and cache.rgbarr is not None:
             return
         dstarr = self._rgbarr2
 
-        t1 = t2 = t3 = t4 = time.time()
-
         self._common_draw(cvs_img, cache, whence)
 
         if cache.cutout is None:
             return
 
-        t2 = time.time()
         if cvs_img.rgbmap is not None:
             rgbmap = cvs_img.rgbmap
         else:
@@ -653,7 +766,6 @@ class StandardPixelRenderer(RendererBase):
             self.logger.debug("shape of index is %s" % (str(idx.shape)))
             cache.prergb = idx
 
-        t3 = time.time()
         dst_order = self.get_rgb_order()
 
         if (whence <= 2.0) or (cache.rgbarr is None) or (not cvs_img.optimize):
@@ -666,8 +778,6 @@ class StandardPixelRenderer(RendererBase):
                 a_idx = dst_order.index('A')
                 cache.rgbarr[..., a_idx] = cache.alpha
 
-        t4 = time.time()
-
         # composite the image into the destination array at the
         # calculated position
         trcalc.overlay_image(dstarr, cache.cvs_pos, cache.rgbarr,
@@ -675,9 +785,6 @@ class StandardPixelRenderer(RendererBase):
                              alpha=cvs_img.alpha, fill=True, flipy=False)
 
         cache.drawn = True
-        t5 = time.time()
-        self.logger.debug("draw: t2=%.4f t3=%.4f t4=%.4f t5=%.4f total=%.4f" % (
-            t2 - t1, t3 - t2, t4 - t3, t5 - t4, t5 - t1))
 
     def _apply_visuals(self, cvs_img, data, vmin, vmax):
         if cvs_img.autocuts is not None:
@@ -852,35 +959,167 @@ class StandardPixelRenderer(RendererBase):
         return font_asst.scale_fontsize(self.kind, fontsize)
 
 
-def get_render_class(rtype):
+class StandardPipelineRenderer(RendererBase):
+    """Standard renderer for generating bitmap-based image that can be
+    copied to an RGB image type-widget or a canvas.
 
-    rtype = rtype.lower()
-    if rtype == 'pil':
-        from ginga.pilw import CanvasRenderPil
-        return CanvasRenderPil.CanvasRenderer
+    This renderer has a pipeline that looks like:
 
-    if rtype == 'agg':
-        from ginga.aggw import CanvasRenderAgg
-        return CanvasRenderAgg.CanvasRenderer
+    [createbg] => [overlays] => [iccprof] => [flipswap] => [rotate] => [output]
 
-    if rtype == 'opencv':
-        from ginga.cvw import CanvasRenderCv
-        return CanvasRenderCv.CanvasRenderer
+    """
 
-    if rtype == 'cairo':
-        from ginga.cairow import CanvasRenderCairo
-        return CanvasRenderCairo.CanvasRenderer
+    def __init__(self, viewer):
+        super(StandardPipelineRenderer, self).__init__(viewer)
 
-    if rtype == 'opengl':
-        from ginga.opengl import CanvasRenderGL
-        return CanvasRenderGL.CanvasRenderer
+        # stages making up the rendering pipeline
+        self.stage = Bunch.Bunch(createbg=render.CreateBg(viewer),
+                                 overlays=render.Overlays(viewer),
+                                 iccprofile=render.ICCProf(viewer),
+                                 flipswap=render.FlipSwap(viewer),
+                                 rotate=render.Rotate(viewer),
+                                 output=render.Output(viewer))
+        # build the pipeline
+        self.pipeline = pipeline.Pipeline(self.logger,
+                                          [self.stage.createbg,
+                                           self.stage.overlays,
+                                           self.stage.iccprofile,
+                                           self.stage.flipswap,
+                                           self.stage.rotate,
+                                           self.stage.output],
+                                          name='standard-pixel-renderer')
 
-    if rtype == 'qt':
-        from ginga.qtw import CanvasRenderQt
-        return CanvasRenderQt.CanvasRenderer
+        # A table of (threshold, stage) tuples. See render_whence()
+        self.tbl = [(2.0, self.stage.overlays),
+                    (2.3, self.stage.iccprofile),
+                    (2.5, self.stage.flipswap),
+                    (2.6, self.stage.rotate),
+                    ]
 
-    if rtype == 'vqt':
-        from ginga.qtw import CanvasRenderQt
-        return CanvasRenderQt.VectorCanvasRenderer
+        # order of RGBA channels that renderer needs to work in
+        self.rgb_order = 'RGBA'
+        self.std_order = 'RGBA'
+        self.dims = (0, 0)
+        self.state = Bunch.Bunch(org_scale=(1.0, 1.0),
+                                 org_pan=(0.0, 0.0),
+                                 ctr=(0, 0),
+                                 win_dim=(0, 0),
+                                 order=self.std_order)
+        self.pipeline.set(state=self.state)
+        # initialize pipeline
+        self.pipeline.invalidate()
 
-    raise ValueError("Don't know about '%s' renderer type" % (rtype))
+    def get_rgb_order(self):
+        return self.rgb_order
+
+    def invalidate(self):
+        self.pipeline.invalidate()
+
+    def initialize(self):
+        # not currently used by this renderer
+        pass
+
+    def render_whence(self, whence):
+        if whence < 3.0:
+            # Search the table and stop at the first stage whose threshold
+            # is <= whence. Run the pipeline from that stage.
+            for thr, stage in self.tbl:
+                if whence <= thr:
+                    self.pipeline.set(whence=whence)
+                    self.pipeline.run_from(stage)
+                    break
+
+        # TODO: what are options for high bit depth under the backend?
+        last_stage = self.pipeline[-1]
+        rgbarr = self.pipeline.get_data(last_stage)
+
+        # NOTE: we assume Output stage has rendered according to the
+        # RGB order needed by the renderer
+        dst_order = self.get_rgb_order()
+        out_order = self.pipeline.get('out_order')
+        if dst_order != out_order:
+            raise RenderError("RGB order of pipeline output ({}) "
+                              "does not match renderer expected ({})".format(
+                                  out_order, dst_order))
+
+        self.render_image(rgbarr, dst_order, (0, 0))
+
+    def finalize(self):
+        # not currently used by this renderer
+        pass
+
+    def prepare_image(self, cvs_img, cache, whence):
+        stage = self.pipeline[1]
+        if cvs_img.kind == 'image':
+            stage._prepare_image(cvs_img, cache, whence)
+        elif cvs_img.kind == 'normimage':
+            stage._prepare_norm_image(cvs_img, cache, whence)
+        else:
+            raise RenderError("I don't know how to render canvas type '{}'".format(cvs_img.kind))
+
+    def getwin_array(self, order='RGBA', alpha=1.0, dtype=None):
+        # NOTE: alpha parameter temporarily ignored for now
+        dst_order = order.upper()
+        #src_order = self.std_order
+        src_order = self.get_rgb_order()
+
+        last_stage = self.pipeline[-1]
+        outarr = self.pipeline.get_data(last_stage)
+
+        # reorder as caller needs it
+        outarr = self.reorder(dst_order, outarr, src_order=src_order)
+        outarr = np.ascontiguousarray(outarr)
+        if dtype is not None:
+            outarr = outarr.astype(dtype, copy=False)
+        return outarr
+
+    def get_scale(self):
+        return self.state.org_scale
+
+    def get_origin(self):
+        return self.state.org_pan
+
+    def get_window_center(self):
+        return self.state.ctr
+
+    def get_window_size(self):
+        return self.state.win_dim
+
+    def resize(self, dims):
+        self._resize(dims)
+        self.pipeline.run_stage_idx(0)
+        self.viewer.redraw(whence=0)
+
+    def calc_const_len(self, clen):
+        # For standard pixel renderer, pixel size is constant
+        return clen
+
+    def _resize(self, dims):
+        self.dims = dims
+        wd, ht = dims[:2]
+        ctr = (wd // 2, ht // 2)
+        self.state.setvals(win_dim=dims[:2], ctr=ctr,
+                           order=self.std_order)
+
+    def _confirm_pan_and_scale(self, scale_x, scale_y, pan_x, pan_y,
+                               win_wd, win_ht):
+        # Sanity check on the scale
+        sx = float(win_wd) / scale_x
+        sy = float(win_ht) / scale_y
+        if (sx < 1.0) or (sy < 1.0):
+            if self.viewer.settings.get('sanity_check_scale', True):
+                raise RenderError("new scale would exceed pixel max; scale unchanged")
+
+        data_off = self.viewer.data_off
+
+        # record location of pan position pixel
+        org_x, org_y = pan_x - data_off, pan_y - data_off
+
+        org_scale_x, org_scale_y = scale_x, scale_y
+        org_scale_z = (scale_x + scale_y) / 2.0
+
+        self.state.setvals(org_pan=(org_x, org_y, 0.0),
+                           org_scale=(org_scale_x, org_scale_y, org_scale_z))
+
+    def scale_fontsize(self, fontsize):
+        return font_asst.scale_fontsize(self.kind, fontsize)
