@@ -16,7 +16,7 @@ from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
 from ginga.misc.ParamSet import Param
 from ginga.misc import Bunch
 from ginga.util import wcs
-from ginga.util.wcs import raDegToString, decDegToString
+from ginga.util.wcs import ra_deg_to_str, dec_deg_to_str
 
 from .mixins import (OnePointMixin, TwoPointMixin, OnePointOneRadiusMixin,
                      OnePointTwoRadiusMixin)
@@ -92,7 +92,7 @@ class RulerP(TwoPointMixin, CanvasObjectBase):
                  showcap=True, showplumb=True, showends=False, units='arcmin',
                  font='Sans Serif', fontsize=None, **kwdargs):
         self.kind = 'ruler'
-        points = np.asarray([pt1, pt2], dtype=np.float)
+        points = np.asarray([pt1, pt2], dtype=float)
         CanvasObjectBase.__init__(self, color=color, color2=color2,
                                   alpha=alpha, units=units,
                                   showplumb=showplumb, showends=showends,
@@ -137,10 +137,10 @@ class RulerP(TwoPointMixin, CanvasObjectBase):
                 text.h = self.get_arcmin(res.dh_deg)
                 text.x = self.get_arcmin(res.dx_deg)
                 text.y = self.get_arcmin(res.dy_deg)
-                text.b = ("%s, %s" % (wcs.raDegToString(res.ra_org),
-                                      wcs.decDegToString(res.dec_org)))
-                text.e = ("%s, %s" % (wcs.raDegToString(res.ra_dst),
-                                      wcs.decDegToString(res.dec_dst)))
+                text.b = ("%s, %s" % (wcs.ra_deg_to_str(res.ra_org),
+                                      wcs.dec_deg_to_str(res.dec_org)))
+                text.e = ("%s, %s" % (wcs.ra_deg_to_str(res.ra_dst),
+                                      wcs.dec_deg_to_str(res.dec_dst)))
 
             elif mode == 'degrees':
                 text.h = ("%.8f" % res.dh_deg)
@@ -321,7 +321,7 @@ class CompassP(OnePointOneRadiusMixin, CanvasObjectBase):
                  linewidth=1, fontsize=None, font='Sans Serif',
                  alpha=1.0, linestyle='solid', showcap=True, **kwdargs):
         self.kind = 'compass'
-        points = np.asarray([pt], dtype=np.float)
+        points = np.asarray([pt], dtype=float)
         CanvasObjectBase.__init__(self, ctype=ctype, color=color, alpha=alpha,
                                   linewidth=linewidth, showcap=showcap,
                                   linestyle=linestyle,
@@ -532,7 +532,7 @@ class CrosshairP(OnePointMixin, CanvasObjectBase):
                  fontsize=10.0, font='Sans Serif', fontscale=True,
                  format='xy', **kwdargs):
         self.kind = 'crosshair'
-        points = np.asarray([pt], dtype=np.float)
+        points = np.asarray([pt], dtype=float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, linestyle=linestyle,
                                   text=text, textcolor=textcolor,
@@ -720,7 +720,7 @@ class AnnulusP(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
                      coord=coord)
         obj2.editable = False
 
-        points = np.asarray([pt], dtype=np.float)
+        points = np.asarray([pt], dtype=float)
 
         CompoundObject.__init__(self, obj1, obj2,
                                 points=points, radius=radius,
@@ -905,7 +905,7 @@ class Annulus2RP(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
                      coord=coord, rot_deg=rot_deg)
         obj2.editable = False
 
-        points = np.asarray([pt], dtype=np.float)
+        points = np.asarray([pt], dtype=float)
 
         CompoundObject.__init__(self, obj1, obj2,
                                 points=points, xradius=xradius, yradius=yradius,
@@ -1142,12 +1142,12 @@ class WCSAxes(CompoundObject):
         for cur_ra in ra_arr:
             crds = [[cur_ra, cur_dec] for cur_dec in
                     np.arange(dec_min, dec_max + d_dec_step, d_dec_step)]
-            lbl = raDegToString(cur_ra)
+            lbl = ra_deg_to_str(cur_ra)
             objs += self._get_path(viewer, image, crds, lbl, 1)
         for cur_dec in dec_arr:
             crds = [[cur_ra, cur_dec] for cur_ra in
                     np.arange(ra_min, ra_max + d_ra_step, d_ra_step)]
-            lbl = decDegToString(cur_dec)
+            lbl = dec_deg_to_str(cur_dec)
             objs += self._get_path(viewer, image, crds, lbl, 0)
 
         return objs
