@@ -5,6 +5,7 @@
 # Please see the file LICENSE.txt for details.
 #
 import uuid
+import weakref
 import numpy as np
 
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
@@ -71,7 +72,7 @@ class ImageP(OnePointMixin, CanvasObjectBase):
                  showcap=False, flipy=False, optimize=True,
                  **kwdargs):
         self.kind = 'image'
-        points = np.asarray([pt], dtype=np.float)
+        points = np.asarray([pt], dtype=float)
         CanvasObjectBase.__init__(self, points=points, image=image, alpha=alpha,
                                   scale_x=scale_x, scale_y=scale_y,
                                   interpolation=interpolation,
@@ -83,7 +84,7 @@ class ImageP(OnePointMixin, CanvasObjectBase):
 
         # The cache holds intermediate step results by viewer.
         # Depending on value of `whence` they may not need to be recomputed.
-        self._cache = {}
+        self._cache = weakref.WeakKeyDictionary()
         self.image_id = str(uuid.uuid4())
         # images are not editable by default
         self.editable = False
@@ -119,12 +120,15 @@ class ImageP(OnePointMixin, CanvasObjectBase):
         cache = self.get_cache(viewer)
 
         if not cache.drawn:
+            # Normally, prepare_image() will be called earlier, and
+            # cache.drawn will be true.  This catches the case for
+            # some renderers where that might not happen
             viewer.prepare_image(self, cache, whence)
 
         cpoints = self.get_cpoints(viewer)
         cr = viewer.renderer.setup_cr(self)
 
-        cr.draw_image(self, cpoints, cache.rgbarr, whence)
+        cr.draw_image(self, cpoints, cache, whence)
 
         # draw optional border
         if self.linewidth > 0:
@@ -155,6 +159,8 @@ class ImageP(OnePointMixin, CanvasObjectBase):
         self.make_callback('image-set', image)
 
     def get_scaled_wdht(self):
+        if self.image is None:
+            return (0, 0)
         width = self.image.width * self.scale_x
         height = self.image.height * self.scale_y
         return (width, height)
