@@ -18,7 +18,7 @@ from ginga.util.loader import load_data
 
 class FitsViewer(object):
 
-    def __init__(self, logger):
+    def __init__(self, logger, render='widget'):
         self.logger = logger
         self.drawcolors = colors.get_colors()
         self.dc = get_canvas_types()
@@ -27,14 +27,21 @@ class FitsViewer(object):
 
         self.app = Widgets.Application(logger=logger)
         self.app.add_callback('shutdown', self.quit)
-        self.top = self.app.make_window("Ginga example2")
+        if hasattr(Widgets, 'Page'):
+            self.page = Widgets.Page("Ginga example2")
+            self.app.add_window(self.page)
+            self.top = Widgets.TopLevel("Ginga example2")
+            self.page.add_dialog(self.top)
+        else:
+            self.top = Widgets.TopLevel("Ginga example2")
+            self.app.add_window(self.top)
         self.top.add_callback('close', self.closed)
 
         vbox = Widgets.VBox()
         vbox.set_border_width(2)
         vbox.set_spacing(1)
 
-        fi = Viewers.CanvasView(logger=logger)
+        fi = Viewers.CanvasView(logger=logger, render=render)
         fi.enable_autocuts('on')
         fi.set_autocut_params('zscale')
         fi.enable_autozoom('on')
@@ -78,7 +85,7 @@ class FitsViewer(object):
         self.drawtypes.sort()
 
         fi.set_desired_size(512, 512)
-        iw = Viewers.GingaViewerWidget(viewer=fi)
+        iw = Viewers.GingaScrolledViewerWidget(viewer=fi)
         vbox.add_widget(iw, stretch=1)
 
         self.readout = Widgets.Label("")
@@ -275,23 +282,10 @@ def main(options, args):
     # decide our toolkit, then import
     ginga_toolkit.use(options.toolkit)
 
-    if options.use_opencv:
-        from ginga import trcalc
-        try:
-            trcalc.use('opencv')
-        except Exception as e:
-            logger.warning("Error using OpenCv: %s" % str(e))
+    rw = 'opengl' if options.renderer == 'opengl' else 'widget'
+    viewer = FitsViewer(logger, render=rw)
 
-    if options.use_opencl:
-        from ginga import trcalc
-        try:
-            trcalc.use('opencl')
-        except Exception as e:
-            logger.warning("Error using OpenCL: %s" % str(e))
-
-    viewer = FitsViewer(logger)
-
-    if options.renderer is not None:
+    if options.renderer is not None and options.renderer != 'opengl':
         render_class = render.get_render_class(options.renderer)
         viewer.fitsimage.set_renderer(render_class(viewer.fitsimage))
 
@@ -320,21 +314,9 @@ if __name__ == "__main__":
 
     argprs = ArgumentParser()
 
-    argprs.add_argument("--debug", dest="debug", default=False,
-                        action="store_true",
-                        help="Enter the pdb debugger on main()")
     argprs.add_argument("-t", "--toolkit", dest="toolkit", metavar="NAME",
                         default='qt',
                         help="Choose GUI toolkit (gtk|qt)")
-    argprs.add_argument("--opencv", dest="use_opencv", default=False,
-                        action="store_true",
-                        help="Use OpenCv acceleration")
-    argprs.add_argument("--opencl", dest="use_opencl", default=False,
-                        action="store_true",
-                        help="Use OpenCL acceleration")
-    argprs.add_argument("--profile", dest="profile", action="store_true",
-                        default=False,
-                        help="Run the profiler on main()")
     argprs.add_argument("-r", "--renderer", dest="renderer", metavar="NAME",
                         default=None,
                         help="Choose renderer (pil|agg|opencv|cairo|qt)")
@@ -342,20 +324,4 @@ if __name__ == "__main__":
 
     (options, args) = argprs.parse_known_args(sys.argv[1:])
 
-    # Are we debugging this?
-    if options.debug:
-        import pdb
-
-        pdb.run('main(options, args)')
-
-    # Are we profiling this?
-    elif options.profile:
-        import profile
-
-        print(("%s profile:" % sys.argv[0]))
-        profile.run('main(options, args)')
-
-    else:
-        main(options, args)
-
-# END
+    main(options, args)
