@@ -14,6 +14,7 @@ import numpy as np
 
 from ginga.misc import Bunch, Callback
 from ginga.fonts import font_asst
+import ginga.icons
 import ginga.toolkit
 
 import gi
@@ -27,6 +28,8 @@ import cairo
 
 ginga.toolkit.use('gtk3')
 
+# path to our icons
+icondir = os.path.split(ginga.icons.__file__)[0]
 
 DND_TARGET_TYPE_TEXT = 0
 DND_TARGET_TYPE_URIS = 1
@@ -293,7 +296,7 @@ class MDISubWindow(Callback.Callbacks):
         evbox = Gtk.EventBox()
         evbox.add(label)
         modify_bg(evbox, "gray90")
-        self.label = evbox
+        self.label = label
         self.evbox = evbox
         hbox.pack_start(evbox, True, True, 2)
 
@@ -325,12 +328,23 @@ class MDISubWindow(Callback.Callbacks):
         minim.connect('clicked', lambda *args: self.make_callback('minimize'))
         close.connect('clicked', lambda *args: self.make_callback('close'))
 
+    def raise_(self):
+        window = self.frame.get_window()
+        if window is not None:
+            window.raise_()
+
+    def lower(self):
+        window = self.frame.get_window()
+        if window is not None:
+            window.lower()
+
+    def focus(self):
+        self.frame.grab_focus()
+
 
 class MDIWidget(Gtk.Layout):
     """
     Multiple Document Interface type widget for Gtk.
-
-    NOTE: *** This is somewhat of a work in progress! ***
     """
     def __init__(self):
         Gtk.Layout.__init__(self)
@@ -363,9 +377,8 @@ class MDIWidget(Gtk.Layout):
 
         modify_bg(self, "gray50")
 
-    def append_page(self, widget, label):
+    def add_subwin(self, subwin):
 
-        subwin = MDISubWindow(widget, label)
         self.children.append(subwin)
 
         subwin.evbox.connect("button_press_event", self.select_child_cb, subwin)
@@ -373,15 +386,7 @@ class MDIWidget(Gtk.Layout):
         subwin.add_callback('maximize', lambda *args: self.maximize_page(subwin))
         subwin.add_callback('minimize', lambda *args: self.minimize_page(subwin))
 
-        # pick a random spot to place the window initially
-        rect = self.get_allocation()
-        wd, ht = rect.width, rect.height
-        x = random.randint(self.cascade_offset,
-                           max(self.cascade_offset + 10, wd // 2))
-        y = random.randint(self.cascade_offset,
-                           max(self.cascade_offset + 10, ht // 2))
-
-        self.put(subwin.frame, x, y)
+        self.put(subwin.frame, subwin.x, subwin.y)
 
         # note: seem to need a slight delay to let the widget be mapped
         # in order to accurately determine its position and size
@@ -391,6 +396,21 @@ class MDIWidget(Gtk.Layout):
         GObject.timeout_add(1500, self.update_subwin_size, subwin)
 
         self._update_area_size()
+
+    def append_page(self, widget, label):
+
+        subwin = MDISubWindow(widget, label)
+
+        # pick a random spot to place the window initially
+        rect = self.get_allocation()
+        wd, ht = rect.width, rect.height
+        x = random.randint(self.cascade_offset,  # nosec
+                           max(self.cascade_offset + 10, wd // 2))
+        y = random.randint(self.cascade_offset,  # nosec
+                           max(self.cascade_offset + 10, ht // 2))
+        subwin.x, subwin.y = x, y
+
+        self.add_subwin(subwin)
         return subwin
 
     def set_tab_reorderable(self, w, tf):
@@ -414,9 +434,7 @@ class MDIWidget(Gtk.Layout):
 
     def set_current_page(self, idx):
         subwin = self.children[idx]
-        #frame = subwin.frame
-        #frame.show()
-        self.raise_widget(subwin)
+        subwin.raise_()
         self.cur_index = idx
 
     def get_current_page(self):
@@ -465,12 +483,7 @@ class MDIWidget(Gtk.Layout):
         subwin.width, subwin.height = wd, ht
 
     def raise_widget(self, subwin):
-        frame = subwin.frame
-        # Hack to bring widget to the top--no documentation on any other
-        # way to accomplish this
-        super(MDIWidget, self).remove(frame)
-        frame.unparent()
-        self.put(frame, subwin.x, subwin.y)
+        subwin.raise_()
 
     def select_child_cb(self, layout, event, subwin):
         x_root, y_root = event.x_root, event.y_root
@@ -679,7 +692,7 @@ class MDIWidget(Gtk.Layout):
                     x, y = j * wd, i * ht
                     self.move_page(subwin, x, y)
 
-                    self.raise_widget(subwin)
+                    subwin.raise_()
 
         self._update_area_size()
 
@@ -687,7 +700,7 @@ class MDIWidget(Gtk.Layout):
         x, y = 0, 0
         for subwin in self.children:
             self.move_page(subwin, x, y)
-            self.raise_widget(subwin)
+            subwin.raise_()
             x += self.cascade_offset
             y += self.cascade_offset
 
@@ -708,7 +721,7 @@ class MDIWidget(Gtk.Layout):
         rect = self.get_allocation()
         wd, ht = rect.width, rect.height
 
-        self.raise_widget(subwin)
+        subwin.raise_()
         self.resize_page(subwin, wd, ht)
         self.move_page(subwin, 0, 0)
 
@@ -726,7 +739,7 @@ class MDIWidget(Gtk.Layout):
 
         self.resize_page(subwin, self.minimized_width, ht)
         self.move_page(subwin, x, height - ht)
-        #self.lower_widget(subwin)
+        subwin.lower()
 
         self._update_area_size()
 
@@ -734,6 +747,274 @@ class MDIWidget(Gtk.Layout):
         self._update_area_size()
 
 
+class Splitter(Gtk.Layout):
+    """
+    Splitter type widget for Gtk.
+    """
+    def __init__(self, orientation='horizontal', thumb_px=8):
+        Gtk.Layout.__init__(self)
+
+        self.orientation = orientation
+        self._sizes = []
+        self._dims = (0, 0)
+        self.children = []
+        self.thumbs = []
+        self.thumb_px = thumb_px
+        self.thumb_aspect = 3.25
+        self.kbdmouse_mask = 0
+
+        mask = self.get_events()
+        self.set_events(mask |
+                        Gdk.EventMask.ENTER_NOTIFY_MASK |
+                        Gdk.EventMask.LEAVE_NOTIFY_MASK
+                        )
+
+        self.connect("size-allocate", self._size_allocate_cb)
+        modify_bg(self, "gray50")
+
+    def add_widget(self, widget):
+        rect = self.get_allocation()
+        wd, ht = rect.width, rect.height
+
+        self.children.append(widget)
+
+        if len(self.children) == 1:
+            widget.set_size_request(wd, ht)
+            self.put(widget, 0, 0)
+            sizes = self._sizes
+            if len(sizes) == 0:
+                pos = wd if self.orientation == 'horizontal' else ht
+                sizes = [pos]
+            self.set_sizes(sizes)
+
+        else:
+            if self.orientation == 'horizontal':
+                thumbfile, _w, _h = ('vdots.png', self.thumb_px,
+                                     int(self.thumb_px * self.thumb_aspect))
+            else:
+                thumbfile, _w, _h = ('hdots.png',
+                                     int(self.thumb_px * self.thumb_aspect),
+                                     self.thumb_px)
+            iconfile = os.path.join(icondir, thumbfile)
+            pixbuf = pixbuf_new_from_file_at_size(iconfile, _w, _h)
+            image = Gtk.Image.new_from_pixbuf(pixbuf)
+            thumb = Gtk.EventBox()
+            thumb.set_visible_window(True)
+            thumb.add(image)
+            modify_bg(thumb, "gray90")
+
+            i = len(self.thumbs)
+            self.thumbs.append(thumb)
+            thumb.connect("button_press_event", self._start_resize_cb, i)
+            thumb.connect("button_release_event", self._stop_resize_cb, i)
+            thumb.connect("motion_notify_event", self._do_resize_cb, i)
+            thumb.connect("enter_notify_event", self._thumb_enter_cb)
+            thumb.connect("leave_notify_event", self._thumb_leave_cb)
+
+            self.put(thumb, 0, 0)
+            self.put(widget, 0, 0)
+
+            sizes = self._sizes
+            if len(sizes) < len(self.children):
+                pos = wd if self.orientation == 'horizontal' else ht
+                sizes.append(pos)
+            self.set_sizes(sizes)
+
+        self.show_all()
+
+    def _thumb_enter_cb(self, widget, event):
+        # change the cursor to a resize one when we enter the thumb area
+        display = self.get_display()
+        cur_name = ('ew-resize' if self.orientation == 'horizontal'
+                    else 'ns-resize')
+        cursor = Gdk.Cursor.new_from_name(display, cur_name)
+        win = self.get_window()
+        if win is not None:
+            win.set_cursor(cursor)
+
+    def _thumb_leave_cb(self, widget, event):
+        # change the cursor to the normal one when we leave the thumb area
+        display = self.get_display()
+        cursor = Gdk.Cursor.new_from_name(display, 'default')
+        win = self.get_window()
+        if win is not None:
+            win.set_cursor(cursor)
+
+    def get_sizes(self):
+        return list(self._sizes)
+
+    def set_sizes(self, sizes):
+        sizes = list(sizes)
+        ## if sizes == self._sizes:
+        ##     return
+        if self.get_realized():
+            rect = self.get_allocation()
+            wd, ht = rect.width, rect.height
+        else:
+            min_req, nat_req = self.get_preferred_size()
+            wd, ht = nat_req.width, nat_req.height
+
+        x, y = 0, 0
+        # calc space needed by all necessary thumbs
+        remaining_thumb_space = max(0, len(self.children) - 1) * self.thumb_px
+        new_sizes = []
+        thumbs, widgets = [], []
+        for num, child in enumerate(self.children):
+            off = sizes[num]
+
+            if self.orientation == 'horizontal':
+                if num == 0:
+                    widgets.append((child, 0, 0, off, ht))
+                    new_sizes.append(off)
+                    x += off
+
+                else:
+                    thumb = self.thumbs[num - 1]
+                    thumbs.append((thumb, x, y, self.thumb_px, ht))
+                    x += self.thumb_px
+                    remaining_thumb_space -= self.thumb_px
+
+                    rest = max(0, wd - (x + remaining_thumb_space))
+                    if num < len(self.children) - 1:
+                        rest = min(off, rest)
+                    widgets.append((child, x, y, rest, ht))
+                    new_sizes.append(rest)
+                    x += rest
+
+            else:
+                if num == 0:
+                    widgets.append((child, 0, 0, wd, off))
+                    new_sizes.append(off)
+                    y += off
+
+                else:
+                    thumb = self.thumbs[num - 1]
+                    thumbs.append((thumb, x, y, wd, self.thumb_px))
+                    y += self.thumb_px
+                    remaining_thumb_space -= self.thumb_px
+
+                    rest = max(0, ht - (y + remaining_thumb_space))
+                    if num < len(self.children) - 1:
+                        rest = min(off, rest)
+                    widgets.append((child, x, y, wd, rest))
+                    new_sizes.append(rest)
+                    y += rest
+
+        self._sizes = new_sizes
+        assert len(self._sizes) == len(self.children)
+
+        for child, x, y, wd, ht in widgets:
+            self._move_resize_child(child, x, y, wd, ht)
+        for thumb, x, y, wd, ht in thumbs:
+            self._move_resize_child(thumb, x, y, wd, ht)
+
+    def remove(self, child):
+        if child not in self.children:
+            raise ValueError("widget is not one of our children")
+        idx = self.children.index(child)
+        if len(self.children) > 1:
+            if idx > 0:
+                # not first child
+                thumb = self.thumbs.pop(idx - 1)
+            else:
+                thumb = self.thumbs.pop(0)
+
+            super(Splitter, self).remove(thumb)
+
+        self._sizes.pop(idx)
+        self.children.remove(child)
+        super(Splitter, self).remove(child)
+
+        self.set_sizes(self._sizes)
+
+    def _move_resize_child(self, child, x, y, wd, ht):
+        rect = child.get_allocation()
+        modified = False
+
+        if (rect.x, rect.y) != (x, y):
+            modified = True
+            self.move(child, x, y)
+
+        if (rect.width, rect.height) != (wd, ht):
+            modified = True
+            child.set_size_request(wd, ht)
+
+            alloc = Gdk.Rectangle()
+            alloc.x, alloc.y, alloc.width, alloc.height = x, y, wd, ht
+            child.size_allocate(alloc)
+
+            #child.set_clip(alloc)
+
+            win = child.get_window()
+            if win is not None:
+                win.invalidate_rect(None, True)
+                win.resize(wd, ht)
+
+        if modified:
+            # don't think this should be necessary, but just in case
+            child.queue_draw()
+            #child.queue_resize()
+            child.queue_allocate()
+
+    def _calc_size(self, i, pos):
+        sizes = list(self._sizes)
+        n = sum([sizes[j] for j in range(0, i)])
+        n += max(0, i - 1) * self.thumb_px
+        return max(0, pos - n)
+
+    def _start_resize_cb(self, widget, event, i):
+        x_root, y_root = event.x_root, event.y_root
+        x, y = widget.translate_coordinates(self, event.x, event.y)
+
+        pos = x if self.orientation == 'horizontal' else y
+        sizes = list(self._sizes)
+        sizes[i] = self._calc_size(i, pos)
+        self.set_sizes(sizes)
+        return True
+
+    def _stop_resize_cb(self, widget, event, i):
+        x_root, y_root = event.x_root, event.y_root
+        x, y = widget.translate_coordinates(self, event.x, event.y)
+
+        pos = x if self.orientation == 'horizontal' else y
+        sizes = list(self._sizes)
+        sizes[i] = self._calc_size(i, pos)
+        self.set_sizes(sizes)
+        return True
+
+    def _do_resize_cb(self, widget, event, i):
+        button = self.kbdmouse_mask
+        x_root, y_root, state = event.x_root, event.y_root, event.state
+        x, y = widget.translate_coordinates(self, event.x, event.y)
+
+        if state & Gdk.ModifierType.BUTTON1_MASK:
+            button |= 0x1
+        elif state & Gdk.ModifierType.BUTTON2_MASK:
+            button |= 0x2
+        elif state & Gdk.ModifierType.BUTTON3_MASK:
+            button |= 0x4
+
+        if button == 0x1:
+            pos = x if self.orientation == 'horizontal' else y
+            sizes = list(self._sizes)
+            sizes[i] = self._calc_size(i, pos)
+            self.set_sizes(sizes)
+        return True
+
+    def _size_allocate_cb(self, widget, rect):
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+
+        dims = (wd, ht)
+        if dims == self._dims:
+            return
+        super(Splitter, self).set_size(wd, ht)
+
+        self._dims = dims
+
+        self.set_sizes(self._sizes)
+        return True
+
+
 class Dial(Gtk.DrawingArea):
 
     __gtype_name__ = "Dial"
