@@ -5,9 +5,10 @@
 # Please see the file LICENSE.txt for details.
 #
 
-# import time
+import os.path
 
 from ginga.gtk3w import GtkHelp
+import ginga.icons
 
 from ginga.misc import Callback, Bunch, Settings, LineHistory
 from functools import reduce
@@ -27,8 +28,11 @@ try:
     from gi.repository import WebKit2 as WebKit  # noqa
     has_webkit = True
 except Exception:
-    gi.require_version('WebKit', '3.0')
-    from gi.repository import WebKit  # noqa
+    try:
+        gi.require_version('WebKit', '3.0')
+        from gi.repository import WebKit  # noqa
+    except Exception:
+        pass
 
 __all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
            'TextArea', 'Label', 'Button', 'ComboBox',
@@ -42,6 +46,9 @@ __all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
            'name_mangle', 'make_widget', 'hadjust', 'build_info', 'wrap',
            'has_webkit']
 
+# path to our icons
+icondir = os.path.split(ginga.icons.__file__)[0]
+
 
 class WidgetError(Exception):
     """For errors thrown in this module."""
@@ -95,7 +102,6 @@ class WidgetBase(Callback.Callbacks):
     def get_pos(self):
         rect = self.widget.get_allocation()
         x, y = rect.x, rect.y
-        # width, height = rect.width, rect.height
         return x, y
 
     def get_app(self):
@@ -112,6 +118,9 @@ class WidgetBase(Callback.Callbacks):
     def hide(self):
         self.widget.hide()
 
+    def is_visible(self):
+        return self.widget.get_visible()
+
     def focus(self):
         self.widget.grab_focus()
 
@@ -404,6 +413,12 @@ class Button(WidgetBase):
 
         self.enable_callback('activated')
 
+    def set_text(self, text):
+        self.widget.set_label(text)
+
+    def get_text(self):
+        return self.widget.get_label()
+
     def _cb_redirect(self, *args):
         self.make_callback('activated')
 
@@ -613,12 +628,20 @@ class ScrollBar(WidgetBase):
             self.widget = Gtk.HScrollbar()
         else:
             self.widget = Gtk.VScrollbar()
+        self.widget.set_range(0.0, 100.0)
         self.widget.connect('value-changed', self._cb_redirect)
 
         self.enable_callback('activated')
 
+    def set_value(self, value):
+        flt_val = value * 100.0
+        self.widget.set_value(flt_val)
+
+    def get_value(self):
+        return self.widget.get_value() / 100.0
+
     def _cb_redirect(self, range):
-        val = range.get_value()
+        val = range.get_value() / 100.0
         self.make_callback('activated', val)
 
 
@@ -1033,6 +1056,13 @@ class TreeView(WidgetBase):
         treepath = model.get_path(item)
         self.tv.scroll_to_cell(treepath, use_align=True, row_align=0.5)
 
+    def scroll_to_end(self):
+        model = self.tv.get_model()
+        num_rows = model.iter_n_children()
+        item = model.iter_nth_child(None, num_rows - 1)
+        treepath = model.get_path(item)
+        self.tv.scroll_to_cell(treepath, use_align=True, row_align=0.5)
+
     def sort_on_column(self, i):
         model = self.tv.get_model()
         model.set_sort_column_id(i, Gtk.SortType.ASCENDING)
@@ -1211,6 +1241,16 @@ class Box(ContainerBase):
     def set_spacing(self, val):
         self.widget.set_spacing(val)
 
+    def insert_widget(self, idx, child, stretch=0.0):
+        child_w = child.get_widget()
+        # TODO: can this be made more accurate?
+        expand = (float(stretch) > 0.0)
+        self.widget.pack_start(child_w, expand, True, 0)
+        self.widget.reorder_child(child_w, idx)
+        self.children.insert(idx, child)
+        self.widget.show_all()
+        self.make_callback('widget-added', child)
+
     def add_widget(self, child, stretch=0.0):
         self.add_ref(child)
         child_w = child.get_widget()
@@ -1246,19 +1286,76 @@ class Frame(ContainerBase):
         self.widget.add(child.get_widget())
         self.widget.show_all()
 
+    def set_text(self, text):
+        w = self.get_widget()
+        lbl = w.get_label_widget()
+        lbl.set_text(text)
+
 
 class Expander(ContainerBase):
-    def __init__(self, title=None):
+    r_arrow = None
+    d_arrow = None
+
+    def __init__(self, title=None, notoggle=False):
         super(Expander, self).__init__()
 
-        w = Gtk.Expander(label=title)
-        self.widget = w
+        vbox = VBox()
+        vbox.set_margins(0, 0, 0, 0)
+        vbox.set_spacing(0)
+        self.widget = vbox.get_widget()
+        self._vbox = vbox
+        if Expander.r_arrow is None:
+            iconpath = os.path.join(icondir, 'triangle-right-48.png')
+            Expander.r_arrow = GtkHelp.pixbuf_new_from_file_at_size(iconpath,
+                                                                    12, 12)
+        if Expander.d_arrow is None:
+            iconpath = os.path.join(icondir, 'triangle-down-48.png')
+            Expander.d_arrow = GtkHelp.pixbuf_new_from_file_at_size(iconpath,
+                                                                    12, 12)
+        self._d_arrow = Gtk.Image.new_from_pixbuf(Expander.d_arrow)
+        self._r_arrow = Gtk.Image.new_from_pixbuf(Expander.r_arrow)
 
-    def set_widget(self, child):
-        self.remove_all()
-        self.add_ref(child)
-        self.widget.add(child.get_widget())
-        self.widget.show_all()
+        self.toggle = None
+        if not notoggle:
+            toggle = ToggleButton(title)
+            self.toggle = toggle
+            toggle_w = toggle.get_widget()
+            toggle_w.set_always_show_image(True)
+            r_arrow = Gtk.Image.new_from_pixbuf(Expander.r_arrow)
+            toggle_w.set_image(r_arrow)
+            toggle.add_callback('activated', self._toggle_widget)
+            vbox.add_widget(toggle, stretch=0)
+
+        self.content = None
+
+        for name in ('opened', 'closed'):
+            self.enable_callback(name)
+
+    def set_widget(self, child, stretch=1):
+        if self.content is not None:
+            self.widget.remove(self.content)
+        self.content = child
+
+    def expand(self, tf):
+        children = self._vbox.get_children()
+        if tf:
+            if self.content is None or self.content in children:
+                return
+            if self.toggle is not None:
+                self.toggle.get_widget().set_image(self._d_arrow)
+            self._vbox.add_widget(self.content, stretch=1)
+            self.make_callback('opened')
+
+        else:
+            if self.content is None or self.content not in children:
+                return
+            if self.toggle is not None:
+                self.toggle.get_widget().set_image(self._r_arrow)
+            self._vbox.remove(self.content)
+            self.make_callback('closed')
+
+    def _toggle_widget(self, w, tf):
+        self.expand(tf)
 
 
 class TabWidget(ContainerBase):
@@ -1366,7 +1463,10 @@ class TabWidget(ContainerBase):
         self.widget.set_current_page(idx)
 
     def index_of(self, child):
-        return self.widget.page_num(child.get_widget())
+        widget = child.get_widget()
+        if widget is None:
+            return -1
+        return self.widget.page_num(widget)
 
     def index_to_widget(self, idx):
         """Returns child corresponding to `idx`"""
@@ -1431,27 +1531,11 @@ class MDIWidget(ContainerBase):
 
     def add_widget(self, child, title=''):
         self.add_ref(child)
-        child_w = child.get_widget()
-        label = Gtk.Label(title)
-        subwin = self.mdi_w.append_page(child_w, label)
-        # self.mdi_w.show_all()
-        # attach title to child
-        child.extdata.tab_title = title
-
-        # does child have a previously saved size
-        size = child.extdata.get('mdi_size', None)
-        if size is not None:
-            wd, ht = size
-            self.mdi_w.resize_page(subwin, wd, ht)
-
-        # does child have a previously saved position
-        pos = child.extdata.get('mdi_pos', None)
-        if pos is not None:
-            x, y = pos
-            self.mdi_w.move_page(subwin, x, y)
-
+        subwin = MDIWindow(self, child, title=title)
         subwin.add_callback('close', self._window_close, child)
+
         self.make_callback('widget-added', child)
+        return subwin
 
     def _remove(self, childw, delete=False):
         self.mdi_w.remove(childw)
@@ -1510,6 +1594,84 @@ class MDIWidget(ContainerBase):
         pass
 
 
+class MDIWindow(WidgetBase):
+    def __init__(self, parent, child, title=''):
+        """NOTE: this widget is not meant to be instantiated except *inside*
+        of MDIWidget implementation.
+        """
+        WidgetBase.__init__(self)
+        self.parent = parent
+        mdi_w = parent.mdi_w
+
+        # does child have a previously saved size?
+        size = child.extdata.get('mdi_size', None)
+        if size is not None:
+            wd, ht = size
+            child.resize(wd, ht)
+
+        child_w = child.get_widget()
+        label = Gtk.Label(title)
+        subwin = GtkHelp.MDISubWindow(child_w, label)
+        self.widget = subwin
+        # attach title to child
+        child.extdata.tab_title = title
+
+        self.enable_callback('close')
+        subwin.add_callback('close', self._window_close)
+
+        # does child have a previously saved position?
+        pos = child.extdata.get('mdi_pos', None)
+        if pos is not None:
+            subwin.x, subwin.y = pos
+
+        mdi_w.add_subwin(subwin)
+
+    def get_pos(self):
+        return self.widget.x, self.widget.y
+
+    def raise_(self):
+        self.widget.raise_()
+
+    def lower(self):
+        self.widget.lower()
+
+    def focus(self):
+        self.widget.focus()
+
+    def move(self, x, y):
+        self.parent.mdi_w.move_page(self.widget, x, y)
+
+    def resize(self, wd, ht):
+        self.parent.mdi_w.resize_page(self.widget, wd, ht)
+
+    def maximize(self):
+        self.parent.mdi_w.maximize_page(self.widget)
+
+    def unmaximize(self):
+        raise WidgetError("this call not available for MDIWindow")
+
+    def fullscreen(self):
+        raise WidgetError("this call not available for MDIWindow")
+
+    def unfullscreen(self):
+        raise WidgetError("this call not available for MDIWindow")
+
+    def is_fullscreen(self):
+        raise WidgetError("this call not available for MDIWindow")
+
+    def iconify(self):
+        self.parent.mdi_w.minimize_page(self.widget)
+
+    def uniconify(self):
+        raise WidgetError("this call not available for MDIWindow")
+
+    def set_title(self, title):
+        self.widget.label.set_text(title)
+
+    def _window_close(self, subwin):
+        return self.make_callback('close')
+
+
 class ScrollArea(ContainerBase):
     def __init__(self):
         super(ScrollArea, self).__init__()
@@ -1547,9 +1709,10 @@ class ScrollArea(ContainerBase):
 
 
 class Splitter(ContainerBase):
-    def __init__(self, orientation='horizontal'):
+    def __init__(self, orientation='horizontal', thumb_px=8):
         super(Splitter, self).__init__()
 
+        # thumb_px ignored in this version
         self.orientation = orientation
         self.widget = self._get_pane()
         self.panes = [self.widget]
@@ -1615,6 +1778,35 @@ class Splitter(ContainerBase):
             pane.set_position(pos)
 
 
+class Splitter2(ContainerBase):
+    def __init__(self, orientation='horizontal', thumb_px=8):
+        super(Splitter, self).__init__()
+
+        self.orientation = orientation
+        self.widget = GtkHelp.Splitter(orientation=self.orientation,
+                                       thumb_px=thumb_px)
+
+    def add_widget(self, child):
+        self.add_ref(child)
+        child_w = child.get_widget()
+
+        # without a Frame it can be difficult to see the divider
+        frame_w = Gtk.Frame()
+        #frame_w.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        frame_w.set_shadow_type(Gtk.ShadowType.NONE)
+        frame_w.add(child_w)
+
+        self.widget.add_widget(frame_w)
+        self.widget.show_all()
+        self.make_callback('widget-added', child)
+
+    def get_sizes(self):
+        return self.widget.get_sizes()
+
+    def set_sizes(self, sizes):
+        self.widget.set_sizes(sizes)
+
+
 class GridBox(ContainerBase):
     def __init__(self, rows=1, columns=1):
         super(GridBox, self).__init__()
@@ -1932,64 +2124,61 @@ class TopLevelMixin(object):
 
     def raise_(self):
         window = self.widget.get_window()
-        window.raise_()
+        if window is not None:
+            window.raise_()
 
     def lower(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.lower()
+        if window is not None:
+            window.lower()
 
     def focus(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.focus()
+        if window is not None:
+            window.focus()
 
     def move(self, x, y):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.move(x, y)
+        if window is not None:
+            window.move(x, y)
 
     def maximize(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.maximize()
+        if window is not None:
+            window.maximize()
 
     def unmaximize(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.unmaximize()
+        if window is not None:
+            window.unmaximize()
+
+    def is_maximized(self):
+        window = self.widget.get_window()
+        mask = Gdk.WindowState.MAXIMIZED
+        return window.get_state() & mask != 0
 
     def fullscreen(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.fullscreen()
+        if window is not None:
+            window.fullscreen()
 
     def unfullscreen(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.unfullscreen()
+        if window is not None:
+            window.unfullscreen()
 
     def is_fullscreen(self):
         return self._fullscreen
 
     def iconify(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.iconify()
+        if window is not None:
+            window.iconify()
 
     def uniconify(self):
         window = self.widget.get_window()
-        if window is None:
-            return
-        window.deiconify()
+        if window is not None:
+            window.deiconify()
 
     def set_title(self, title):
         self.widget.set_title(title)
@@ -2289,6 +2478,8 @@ def make_widget(title, wtype):
         w = ProgressBar()
     elif wtype == 'menubar':
         w = Menubar()
+    elif wtype == 'dial':
+        w = Dial()
     else:
         raise ValueError("Bad wtype=%s" % wtype)
     return w
