@@ -67,16 +67,8 @@ class PluginManager(Callback.Callbacks):
             self.logger.info("Plugin '%s' loaded." % name)
 
         except Exception as e:
-            self.logger.error("Failed to load plugin '%s': %s" % (
-                name, str(e)))
-            try:
-                (type, value, tb) = sys.exc_info()
-                tb_str = "\n".join(traceback.format_tb(tb))
-                self.logger.error("Traceback:\n%s" % (tb_str))
-
-            except Exception as e:
-                tb_str = "Traceback information unavailable."
-                self.logger.error(tb_str)
+            self.logger.error("Failed to load plugin '{}': {}".format(
+                name, e), exc_info=True)
             #raise PluginManagerError(e)
 
     def reload_plugin(self, plname, chinfo=None):
@@ -166,8 +158,8 @@ class PluginManager(Callback.Callbacks):
             self.stop_plugin(bnch.pInfo)
 
         except Exception as e:
-            self.logger.error("Error deactivating plugin: %s" % (str(e)))
-            # TODO: log traceback
+            self.logger.error("Error deactivating plugin: {}".format(e),
+                              exc_info=True)
 
         # Set focus to another plugin if one is running, but only if it
         # is a local plugin
@@ -220,6 +212,9 @@ class PluginManager(Callback.Callbacks):
         self.focus.add(lname)
         self.make_callback('focus-plugin', bnch)
 
+        self.raise_plugin(p_info)
+
+    def raise_plugin(self, p_info):
         if p_info.widget is not None:
             self.logger.debug("raising plugin tab %s" % (p_info.tabname))
             if p_info.is_toplevel:
@@ -315,8 +310,7 @@ class PluginManager(Callback.Callbacks):
                     p_info.obj.build_gui(vbox)
 
         except Exception as e:
-            errstr = "Plugin UI failed to initialize: %s" % (
-                str(e))
+            errstr = "Plugin UI failed to initialize: {}".format(e)
             self.logger.error(errstr)
             try:
                 (type, value, tb) = sys.exc_info()
@@ -339,8 +333,7 @@ class PluginManager(Callback.Callbacks):
                 p_info.obj.start()
 
         except Exception as e:
-            errstr = "Plugin failed to start correctly: %s" % (
-                str(e))
+            errstr = "Plugin failed to start correctly: {}".format(e)
             self.logger.error(errstr)
             try:
                 (type, value, tb) = sys.exc_info()
@@ -359,11 +352,15 @@ class PluginManager(Callback.Callbacks):
             self.finish_gui(p_info, vbox)
 
             self.activate(p_info)
+            # focusing plugin will also raise plugin and associated
+            # channel viewer
             self.set_focus(p_info.name)
+
         else:
             self.activate(p_info)
-            # If this is a local plugin, raise the channel associated with the
-            # plug in
+
+            # If this is a local plugin, raise the channel viewer
+            # associated with the plug in
             if p_info.chinfo is not None:
                 itab = p_info.chinfo.name
                 self.ds.raise_tab(itab)
@@ -377,15 +374,8 @@ class PluginManager(Callback.Callbacks):
 
         except Exception as e:
             wasError = True
-            self.logger.error("Plugin failed to stop correctly: %s" % (
-                str(e)))
-            try:
-                (type, value, tb) = sys.exc_info()
-                tb_str = "".join(traceback.format_tb(tb))
-                self.logger.error("Traceback:\n%s" % (tb_str))
-
-            except Exception:
-                self.logger.error("Traceback information unavailable.")
+            self.logger.error("Plugin '{}' failed to stop correctly: {}".format(p_info.name, e),
+                              exc_info=True)
 
         if p_info.widget is not None:
             self.dispose_gui(p_info)
@@ -395,11 +385,13 @@ class PluginManager(Callback.Callbacks):
             raise PluginManagerError(e)
 
     def stop_all_plugins(self):
-        for plugin_name in self.plugin:
+        for plugin_name in self.get_active():
             try:
-                self.stop_plugin(plugin_name)
+                p_info = self.get_plugin_info(plugin_name)
+                self.stop_plugin(p_info)
             except Exception as e:
-                self.logger.error('Exception while calling stop for plugin %s: %s' % (plugin_name, str(e)))
+                self.logger.error("Exception while calling stop for plugin '{}': {}".format(plugin_name, e),
+                                  exc_info=True)
         return True
 
     def plugin_build_error(self, box, text):
