@@ -113,7 +113,7 @@ class RenderContext(render.RenderContextBase):
 
     ##### DRAWING OPERATIONS #####
 
-    def draw_image(self, cvs_img, cp, rgb_arr, whence, order='RGB'):
+    def draw_image(self, cvs_img, cp, cache, whence, order='RGB'):
         """Render the image represented by (rgb_arr) at (cx, cy)
         in the pixel space.
         """
@@ -183,10 +183,10 @@ class RenderContext(render.RenderContextBase):
                                     self.brush, self.pen)
 
 
-class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
+class CanvasRenderer(vec.VectorRenderMixin, render.StandardPipelineRenderer):
 
     def __init__(self, viewer):
-        render.StandardPixelRenderer.__init__(self, viewer)
+        render.StandardPipelineRenderer.__init__(self, viewer)
         vec.VectorRenderMixin.__init__(self)
 
         self.kind = 'opengl'
@@ -227,6 +227,17 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
         scales = self.viewer.get_scale_xy()
         self.scale(scales)
 
+    def _overlay_images(self, canvas, whence=0.0):
+        #if not canvas.is_compound():
+        if not hasattr(canvas, 'objects'):
+            return
+
+        for obj in canvas.get_objects():
+            if hasattr(obj, 'prepare_image'):
+                obj.prepare_image(self.viewer, whence)
+            elif obj.is_compound() and (obj != canvas):
+                self._overlay_images(obj, whence=whence)
+
     def resize(self, dims):
         """Resize our drawing area to encompass a space defined by the
         given dimensions.
@@ -293,7 +304,7 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
         self.viewer.redraw(whence=0.1)
 
     def interpolation_change(self, interp):
-        self.logger.warning("Interpolation setting currently not supported fully by OpenGL renderer--defaulting to nearest neighbor")
+        self.viewer.redraw(whence=0.0)
 
     def _common_draw(self, cvs_img, cache, whence):
         # internal common drawing phase for all images
@@ -327,6 +338,21 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
                                            method=interp)
             data = res.data
 
+            if interp in ('basic', 'nearest'):
+                cache.interp = 0
+            elif interp in ('linear', 'bilinear'):
+                cache.interp = 1
+            elif interp in ('cubic', 'bicubic'):
+                cache.interp = 2
+            elif interp == 'lanczos':
+                # TODO
+                self.logger.warning("'lanczos' interpolation not yet implemented"
+                                    " for this renderer--using 'bicubic' instead")
+                #cache.interp = 3
+                cache.interp = 2
+            else:
+                cache.interp = 0
+
             # We are limited by maximum texture size supported for the
             # OpenGl implementation.  Images larger than the maximum
             # in any dimension need to be downsampled to fit.
@@ -668,10 +694,6 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
         gl.glBindBuffer(gl.GL_TEXTURE_BUFFER, 0)
         self.logger.debug("uploaded cmap as texture buffer {}".format(map_id))
 
-    def gl_set_image_interpolation(self, interp):
-        # TODO
-        pass
-
     def gl_draw_image(self, cvs_img, cp):
         if not self._drawing:
             # this test ensures that we are not trying to draw before
@@ -705,6 +727,9 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
         _loc = self.pgm_mgr.get_uniform_loc("image_type")
         gl.glUniform1i(_loc, cache.image_type)
 
+        _loc = self.pgm_mgr.get_uniform_loc("interp")
+        gl.glUniform1i(_loc, cache.interp)
+
         # if image has fixed cut levels, use those
         cuts = getattr(cvs_img, 'cuts', None)
         if cuts is not None:
@@ -722,14 +747,10 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
         vertices = trcalc.pad_z(cp, dtype=np.float32)
 
         # Send the data over to the buffer
-        # NOTE: we swap elements 0 and 1, because we will also swap
-        # vertices 0 and 1, this allows us to draw two triangles to complete
-        # the image
-        texcoord = np.array([(1.0, 0.0), (0.0, 0.0),
+        texcoord = np.array([(0.0, 0.0), (1.0, 0.0),
                              (1.0, 1.0), (0.0, 1.0)], dtype=np.float32)
-        # swap vertices of rows 0 and 1
-        vertices[[0, 1]] = vertices[[1, 0]]
         data = np.concatenate((vertices, texcoord), axis=1)
+
         gl.glBindBuffer(gl.GL_ARRAY_BUFFER, self.vbo_img)
         # see https://www.khronos.org/opengl/wiki/Buffer_Object_Streaming
         #gl.glBufferData(gl.GL_ARRAY_BUFFER, None, gl.GL_DYNAMIC_DRAW)
@@ -737,9 +758,7 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
 
         gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
 
-        # See NOTE above
-        gl.glDrawArrays(gl.GL_TRIANGLES, 0, 3)
-        gl.glDrawArrays(gl.GL_TRIANGLES, 1, 4)
+        gl.glDrawArrays(gl.GL_TRIANGLE_FAN, 0, 4)
 
         gl.glBindVertexArray(0)
         gl.glBindBuffer(gl.GL_ARRAY_BUFFER, 0)
@@ -795,7 +814,10 @@ class CanvasRenderer(vec.VectorRenderMixin, render.StandardPixelRenderer):
 
             # draw outline
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_LINE)
-            gl.glLineWidth(pen.linewidth)
+            # > 1.0 not guaranteed to be supported as of OpenGL 4.2
+            # TODO
+            # gl.glLineWidth(pen.linewidth)
+            gl.glLineWidth(1.0)
 
             gl.glDrawArrays(gl_shape, 0, len(vertices))
 
