@@ -137,10 +137,10 @@ class RenderContext(render.RenderContextBase):
         self.cr.path(cpoints, self.pen)
 
 
-class CanvasRenderer(render.StandardPixelRenderer):
+class CanvasRenderer(render.StandardPipelineRenderer):
 
     def __init__(self, viewer):
-        render.StandardPixelRenderer.__init__(self, viewer)
+        render.StandardPipelineRenderer.__init__(self, viewer)
 
         self.kind = 'pil'
         self.rgb_order = 'RGBA'
@@ -166,8 +166,8 @@ class CanvasRenderer(render.StandardPixelRenderer):
     ##     cr = RenderContext(self, self.viewer, self.surface)
     ##     self.draw_vector(cr)
 
-    def render_image(self, rgbobj, dst_x, dst_y):
-        """Render the image represented by (rgbobj) at dst_x, dst_y
+    def render_image(self, rgb_arr, order, win_coord):
+        """Render the image represented by (data) at (win_coord)
         in the pixel space.
         *** internal method-- do not use ***
         """
@@ -177,7 +177,6 @@ class CanvasRenderer(render.StandardPixelRenderer):
 
         # get window contents as a buffer and paste it into the PIL surface
         # TODO: allow greater bit depths when support is better in PIL
-        rgb_arr = self.viewer.getwin_array(order=self.rgb_order, dtype=np.uint8)
         p_image = Image.fromarray(rgb_arr)
 
         if self.surface is None or p_image.size != self.surface.size:
@@ -198,7 +197,7 @@ class CanvasRenderer(render.StandardPixelRenderer):
         wd, ht = self.dims[:2]
 
         # Get PIL surface as a numpy array
-        arr8 = np.fromstring(self.surface.tobytes(), dtype=np.uint8)
+        arr8 = np.frombuffer(self.surface.tobytes(), dtype=np.uint8)
         arr8 = arr8.reshape((ht, wd, 3))
 
         # adjust according to viewer's needed order
