@@ -127,9 +127,9 @@ class RenderContext(render.RenderContextBase):
 
     ##### DRAWING OPERATIONS #####
 
-    def draw_image(self, cvs_img, cpoints, rgb_arr, whence, order='RGBA'):
-        # no-op for this renderer
-        pass
+    ## def draw_image(self, cvs_img, cpoints, cache, whence, order='RGBA'):
+    ##     # no-op for this renderer
+    ##     pass
 
     def draw_text(self, cx, cy, text, rot_deg=0.0):
         self.cr.save()
@@ -185,10 +185,10 @@ class RenderContext(render.RenderContextBase):
         self.cr.drawLines(pts)
 
 
-class CanvasRenderer(render.StandardPixelRenderer):
+class CanvasRenderer(render.StandardPipelineRenderer):
 
     def __init__(self, viewer, surface_type='qimage'):
-        render.StandardPixelRenderer.__init__(self, viewer)
+        render.StandardPipelineRenderer.__init__(self, viewer)
 
         self.kind = 'qt'
         # Qt needs this to be in BGRA
@@ -258,19 +258,20 @@ class CanvasRenderer(render.StandardPixelRenderer):
         # adjust according to viewer's needed order
         return self.reorder(order, arr)
 
-    def render_image(self, rgbobj, win_x, win_y):
-        """Render the image represented by (rgbobj) at win_x, win_y
+    def render_image(self, data, order, win_coord):
+        """Render the image represented by (data) at (win_coord)
         in the pixel space.
         *** internal method-- do not use ***
         """
-        self.logger.debug("redraw surface=%s" % (self.surface))
+        self.logger.debug("redraw surface")
         if self.surface is None:
             return
         self.logger.debug("drawing to surface")
 
+        if data is None:
+            return
+        win_x, win_y = win_coord
         # Prepare array for rendering
-        # TODO: what are options for high bit depth under Qt?
-        data = rgbobj.get_array(self.rgb_order, dtype=np.uint8)
 
         daht, dawd, depth = data.shape
         self.logger.debug("data shape is %dx%dx%d" % (dawd, daht, depth))
@@ -281,12 +282,13 @@ class CanvasRenderer(render.StandardPixelRenderer):
 
         painter = get_painter(drawable)
 
-        # fill surface with background color
-        size = drawable.size()
-        sf_wd, sf_ht = size.width(), size.height()
-        bg = self.viewer.img_bg
-        bgclr = self._get_color(*bg)
-        painter.fillRect(QtCore.QRect(0, 0, sf_wd, sf_ht), bgclr)
+        # fill surface with background color (not-strictly necessary,
+        # since image should cover entire window, but to be safe, I guess...)
+        ## size = drawable.size()
+        ## sf_wd, sf_ht = size.width(), size.height()
+        ## bg = self.viewer.img_bg
+        ## bgclr = self._get_color(*bg)
+        ## painter.fillRect(QtCore.QRect(0, 0, sf_wd, sf_ht), bgclr)
 
         # draw image data from buffer to offscreen pixmap
         painter.drawImage(QtCore.QRect(win_x, win_y, dawd, daht),
