@@ -6,6 +6,7 @@
 #
 import os
 import tempfile
+from functools import partial
 
 import numpy as np
 
@@ -41,6 +42,7 @@ class RenderGraphicsView(QtGui.QGraphicsView):
         super(RenderGraphicsView, self).__init__(*args, **kwdargs)
 
         self.viewer = None
+        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
 
     def drawBackground(self, painter, rect):
         """When an area of the window is exposed, we just copy out of the
@@ -78,6 +80,7 @@ class RenderWidget(QtGui.QWidget):
         super(RenderWidget, self).__init__(*args, **kwdargs)
 
         self.viewer = None
+        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
         self.setAttribute(QtCore.Qt.WA_OpaquePaintEvent)
 
     def paintEvent(self, event):
@@ -119,11 +122,20 @@ class RenderWidget(QtGui.QWidget):
 
 class RenderGLWidget(QOpenGLWidget):
 
+    @staticmethod
+    def _on_destroyed(d):
+        viewer, d['viewer'] = d['viewer'], None
+        viewer.imgwin = None
+
     def __init__(self, *args, **kwdargs):
         QOpenGLWidget.__init__(self, *args, **kwdargs)
 
         self.viewer = None
 
+        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
+        self.destroyed.connect(partial(RenderGLWidget._on_destroyed,
+                                       self.__dict__))
+
         # ensure we are using correct version of opengl
         fmt = QSurfaceFormat()
         fmt.setVersion(req.major, req.minor)
@@ -562,6 +574,8 @@ class QtEventMixin(object):
         width = x2 - x1 + 1
         height = y2 - y1 + 1
 
+        self.switch_cursor('pick')
+
         self.configure_window(width, height)
         return self.make_callback('map')
 
@@ -906,6 +920,7 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         self.viewer = viewer
         super(ScrolledView, self).__init__(parent=parent)
 
+        self._bar_status = dict(horizontal='on', vertical='on')
         # the window jiggles annoyingly as the scrollbar is alternately
         # shown and hidden if we use the default "as needed" policy, so
         # default to always showing them (user can change this after
@@ -1025,6 +1040,8 @@ class ScrolledView(QtGui.QAbstractScrollArea):
             self._scrolling = False
 
     def scroll_bars(self, horizontal='on', vertical='on'):
+        self._bar_status.update(dict(horizontal=horizontal,
+                                     vertical=vertical))
         if horizontal == 'on':
             self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
         elif horizontal == 'off':
@@ -1043,6 +1060,9 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         else:
             raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (vertical))
 
+    def get_scroll_bars_status(self):
+        return self._bar_status
+
 
 def save_debug_image(qimage, filename, format='png'):
     path = os.path.join(tempfile.gettempdir(), filename)
