@@ -5,6 +5,7 @@
 # Please see the file LICENSE.txt for details.
 #
 import os.path
+import pathlib
 from functools import reduce
 
 from ginga.qtw.QtHelp import (QtGui, QtCore, QTextCursor, QIcon, QPixmap,
@@ -53,7 +54,6 @@ class WidgetBase(Callback.Callbacks):
         super(WidgetBase, self).__init__()
 
         self.widget = None
-        self.changed = False
         # external data can be attached here
         self.extdata = Bunch.Bunch()
 
@@ -98,6 +98,9 @@ class WidgetBase(Callback.Callbacks):
     def hide(self):
         self.widget.hide()
 
+    def is_visible(self):
+        return self.widget.isVisible()
+
     def get_font(self, font_family, point_size):
         font = QtHelp.get_font(font_family, point_size)
         return font
@@ -341,6 +344,10 @@ class Label(WidgetBase):
         self.widget.setStyleSheet(
             "QLabel { background-color: %s; color: %s; }" % (bg, fg))
 
+    def set_halign(self, align):
+        # TODO: set horizontal alignment of text
+        pass
+
 
 class Button(WidgetBase):
     def __init__(self, text=''):
@@ -351,6 +358,12 @@ class Button(WidgetBase):
 
         self.enable_callback('activated')
 
+    def set_text(self, text):
+        self.widget.setText(text)
+
+    def get_text(self):
+        return self.widget.text()
+
     def _cb_redirect(self, *args):
         self.make_callback('activated')
 
@@ -393,11 +406,13 @@ class ComboBox(WidgetBase):
         self.widget.clear()
 
     def set_text(self, text):
+        self.widget.blockSignals(True)
         index = self.widget.findText(text)
         if index >= 0:
             self.set_index(index)
         else:
             self.widget.setEditText(text)
+        self.widget.blockSignals(False)
 
     # to be deprecated someday
     show_text = set_text
@@ -410,7 +425,9 @@ class ComboBox(WidgetBase):
         self.widget.addItem(text)
 
     def set_index(self, index):
+        self.widget.blockSignals(True)
         self.widget.setCurrentIndex(index)
+        self.widget.blockSignals(False)
 
     def get_index(self):
         return self.widget.currentIndex()
@@ -432,17 +449,15 @@ class SpinBox(WidgetBase):
         self.enable_callback('value-changed')
 
     def _cb_redirect(self, val):
-        if self.changed:
-            self.changed = False
-            return
         self.make_callback('value-changed', val)
 
     def get_value(self):
         return self.widget.value()
 
     def set_value(self, val):
-        self.changed = True
+        self.widget.blockSignals(True)
         self.widget.setValue(val)
+        self.widget.blockSignals(False)
 
     def set_decimals(self, num):
         if hasattr(self.widget, 'setDecimals'):
@@ -477,21 +492,15 @@ class Slider(WidgetBase):
         self.enable_callback('value-changed')
 
     def _cb_redirect(self, val):
-        # It appears that Qt uses set_value() to set the value of the
-        # slider when it is dragged, so we cannot use the usual method
-        # of setting a hidden "changed" variable to suppress the callback
-        # when setting the value programmatically.
-        # if self.changed:
-        #     self.changed = False
-        #     return
         self.make_callback('value-changed', val)
 
     def get_value(self):
         return self.widget.value()
 
     def set_value(self, val):
-        self.changed = True
+        self.widget.blockSignals(True)
         self.widget.setValue(val)
+        self.widget.blockSignals(False)
 
     def set_tracking(self, tf):
         self.widget.setTracking(tf)
@@ -527,13 +536,6 @@ class Dial(WidgetBase):
         self.enable_callback('value-changed')
 
     def _cb_redirect(self, val):
-        # It appears that Qt uses set_value() to set the value of the
-        # slider when it is dragged, so we cannot use the usual method
-        # of setting a hidden "changed" variable to suppress the callback
-        # when setting the value programmatically.
-        # if self.changed:
-        #     self.changed = False
-        #     return
         val = self.get_value()
         self.make_callback('value-changed', val)
 
@@ -558,7 +560,9 @@ class Dial(WidgetBase):
             raise ValueError("Value '{}' is out of range".format(val))
         self.changed = True
         int_val = self._cvt_value_in(val)
+        self.widget.blockSignals(True)
         self.widget.setValue(int_val)
+        self.widget.blockSignals(False)
 
     def set_tracking(self, tf):
         self.widget.setTracking(tf)
@@ -584,8 +588,15 @@ class ScrollBar(WidgetBase):
 
         self.enable_callback('activated')
 
+    def set_value(self, value):
+        int_val = int(round(value * 100.0))
+        self.widget.setValue(int_val)
+
+    def get_value(self):
+        return self.widget.value() / 100.0
+
     def _cb_redirect(self):
-        val = self.widget.value()
+        val = self.widget.value() / 100.0
         self.make_callback('activated', val)
 
 
@@ -603,7 +614,9 @@ class CheckBox(WidgetBase):
         self.make_callback('activated', val)
 
     def set_state(self, tf):
+        self.widget.blockSignals(True)
         self.widget.setChecked(tf)
+        self.widget.blockSignals(False)
 
     def get_state(self):
         val = self.widget.checkState()
@@ -625,7 +638,9 @@ class ToggleButton(WidgetBase):
         self.make_callback('activated', val)
 
     def set_state(self, tf):
+        self.widget.blockSignals(True)
         self.widget.setChecked(tf)
+        self.widget.blockSignals(False)
 
     def get_state(self):
         return self.widget.isChecked()
@@ -641,16 +656,14 @@ class RadioButton(WidgetBase):
         self.enable_callback('activated')
 
     def _cb_redirect(self, val):
-        if self.changed:
-            self.changed = False
-            return
         self.make_callback('activated', val)
 
     def set_state(self, tf):
         if self.widget.isChecked() != tf:
             # toggled only fires when the value is toggled
-            self.changed = True
+            self.widget.blockSignals(True)
             self.widget.setChecked(tf)
+            self.widget.blockSignals(False)
 
     def get_state(self):
         return self.widget.isChecked()
@@ -957,13 +970,15 @@ class TreeView(WidgetBase):
             item.setFont(i, font)
 
     def scroll_to_path(self, path):
-        # TODO: this doesn't give an error, but does not seem to be
-        # working as the API indicates
         item = self._path_to_item(path)
-        row = self.widget.indexOfTopLevelItem(item)
-        midx = self.widget.indexAt(QtCore.QPoint(row, 0))
+        midx = self.widget.indexFromItem(item, 0)
         self.widget.scrollTo(midx, QtGui.QAbstractItemView.PositionAtCenter)
 
+    def scroll_to_end(self):
+        model = self.widget.model()
+        midx = model.index(model.rowCount() - 1, 0)
+        self.widget.scrollTo(midx, QtGui.QAbstractItemView.PositionAtBottom)
+
     def sort_on_column(self, i):
         self.widget.sortByColumn(i, QtCore.Qt.AscendingOrder)
 
@@ -1094,6 +1109,12 @@ class Box(ContainerBase):
         self.layout.setContentsMargins(0, 0, 0, 0)
         self.widget.setLayout(self.layout)
 
+    def insert_widget(self, idx, child, stretch=0.0):
+        self.add_ref(child)
+        child_w = child.get_widget()
+        self.layout.insertWidget(idx, child_w, stretch=stretch)
+        self.make_callback('widget-added', child)
+
     def add_widget(self, child, stretch=0.0):
         self.add_ref(child)
         child_w = child.get_widget()
@@ -1139,6 +1160,10 @@ class Frame(ContainerBase):
         self.add_ref(child)
         self.widget.layout().addWidget(child.get_widget(), stretch=stretch)
 
+    def set_text(self, text):
+        if self.label is not None:
+            self.label.setText(text)
+
 
 # Qt custom expander widget
 # See http://stackoverflow.com/questions/10364589/equivalent-of-gtks-expander-in-pyqt4  # noqa
@@ -1153,7 +1178,7 @@ class Expander(ContainerBase):
                   border-width: 0px; border-style: solid; }
     """
 
-    def __init__(self, title=''):
+    def __init__(self, title='', notoggle=False):
         super(Expander, self).__init__()
 
         # Qt doesn't seem to like it (segfault) if we actually construct
@@ -1171,14 +1196,21 @@ class Expander(ContainerBase):
         vbox.setSpacing(0)
         self.layout = vbox
 
-        self.toggle = QtGui.QPushButton(Expander.r_arrow, title)
-        self.toggle.setStyleSheet(Expander.widget_style)
-        # self.toggle.setCheckable(True)
-        self.toggle.clicked.connect(self._toggle_widget)
+        self.toggle = None
+        if not notoggle:
+            toggle = ToggleButton(title)
+            self.toggle = toggle
+            toggle_w = toggle.get_widget()
+            toggle_w.setIcon(Expander.r_arrow)
+            toggle_w.setStyleSheet(Expander.widget_style)
+            toggle.add_callback('activated', self._toggle_widget)
+            vbox.addWidget(toggle.get_widget(), stretch=0)
 
-        vbox.addWidget(self.toggle, stretch=0)
         self.widget.setLayout(vbox)
 
+        for name in ('opened', 'closed'):
+            self.enable_callback(name)
+
     def set_widget(self, child, stretch=1):
         self.remove_all()
         self.add_ref(child)
@@ -1186,16 +1218,28 @@ class Expander(ContainerBase):
         self.widget.layout().addWidget(child_w, stretch=stretch)
         child_w.setVisible(False)
 
-    def _toggle_widget(self):
+    def expand(self, tf):
         child = self.get_children()[0]
         child_w = child.get_widget()
-        # if self.toggle.isChecked():
-        if child_w.isVisible():
-            self.toggle.setIcon(Expander.r_arrow)
-            child_w.setVisible(False)
-        else:
-            self.toggle.setIcon(Expander.d_arrow)
+        if tf:
+            if child_w.isVisible():
+                # child already open
+                return
+            if self.toggle is not None:
+                self.toggle.get_widget().setIcon(Expander.d_arrow)
             child_w.setVisible(True)
+            self.make_callback('opened')
+        else:
+            if not child_w.isVisible():
+                # child already closed
+                return
+            if self.toggle is not None:
+                self.toggle.get_widget().setIcon(Expander.r_arrow)
+            child_w.setVisible(False)
+            self.make_callback('closed')
+
+    def _toggle_widget(self, w, tf):
+        self.expand(tf)
 
 
 class TabWidget(ContainerBase):
@@ -1382,79 +1426,16 @@ class MDIWidget(ContainerBase):
                 self.cur_index = -1
             self.make_callback('page-switch', child)
 
-    def _window_resized(self, event, subwin, widget):
-        qsize = event.size()
-        wd, ht = qsize.width(), qsize.height()
-        # save size
-        widget.extdata.mdi_size = (wd, ht)
-        subwin._resizeEvent(event)
-
-    def _window_moved(self, event, subwin, widget):
-        qpos = event.pos()
-        x, y = qpos.x(), qpos.y()
-        # save position
-        widget.extdata.mdi_pos = (x, y)
-        subwin._moveEvent(event)
-
-    def _window_closed(self, event, subwin, widget):
-        nchild = subwin.widget()
-        child = self._native_to_child(nchild)
-
-        # let the application deal with this if desired in page-close
-        # callback
-        event.ignore()
-        # self.widget.removeSubWindow(subwin)
-
+    def _child_close(self, subwin, child):
         self.make_callback('page-close', child)
 
     def add_widget(self, child, title=''):
         self.add_ref(child)
-        child_w = child.get_widget()
-        subwin = QtGui.QMdiSubWindow(self.widget)
-        # NOTE: we fire the page-switch callback by intercepting the
-        # focus event on the subwindow, rather than off of the
-        # subWindowActivated signal because the latter fires if
-        # the widget accepts focus when the mouse enters the window,
-        # whereas this approach one actually has to click in the window
-        # or title bar.
+        subwin = MDIWindow(self, child, title=title)
+        subwin.add_callback('close', self._child_close, child)
 
-        def _focus_cb(event):
-            if event.gotFocus():
-                self._cb_redirect(subwin)
-
-        subwin.focusInEvent = _focus_cb
-        # remove Qt logo from subwindow
-        subwin.setWindowIcon(QIcon(QPixmap(1, 1)))
-        subwin.setWidget(child_w)
-        # attach title to child
-        child.extdata.tab_title = title
-
-        w = self.widget.addSubWindow(subwin)
-        w._closeEvent = w.closeEvent
-        w.closeEvent = lambda event: self._window_closed(event, w, child)
-
-        # does child have a previously saved size
-        size = child.extdata.get('mdi_size', None)
-        if size is not None:
-            wd, ht = size
-            w.resize(wd, ht)
-
-        # does child have a previously saved position
-        pos = child.extdata.get('mdi_pos', None)
-        if pos is not None:
-            x, y = pos
-            w.move(x, y)
-
-        # Monkey-patching the widget to take control of resize and move
-        # events
-        w._resizeEvent = w.resizeEvent
-        w.resizeEvent = lambda event: self._window_resized(event, w, child)
-        w._moveEvent = w.moveEvent
-        w.moveEvent = lambda event: self._window_moved(event, w, child)
-        w.setWindowTitle(title)
-        child_w.show()
-        w.show()
         self.make_callback('widget-added', child)
+        return subwin
 
     def _remove(self, nchild, delete=False):
         subwins = list(self.widget.subWindowList())
@@ -1549,15 +1530,32 @@ class ScrollArea(ContainerBase):
 
 
 class Splitter(ContainerBase):
-    def __init__(self, orientation='horizontal'):
+    def __init__(self, orientation='horizontal', thumb_px=8):
         super(Splitter, self).__init__()
 
+        self.thumb_px = thumb_px
+
         w = QtGui.QSplitter()
         self.orientation = orientation
-        if orientation == 'horizontal':
+        # NOTE: need to style splitter due to lack of any visual
+        # indicator on Linux and Windows
+        if self.orientation == 'horizontal':
             w.setOrientation(QtCore.Qt.Horizontal)
+            if thumb_px is not None:
+                iconfile = pathlib.Path(icondir) / 'vdots.png'
+                w.setStyleSheet(
+                    """
+                    QSplitter::handle { width: %spx; height: %spx;
+                                        image: url(%s); }
+                    """ % (self.thumb_px, self.thumb_px, iconfile))
         else:
             w.setOrientation(QtCore.Qt.Vertical)
+            if thumb_px is not None:
+                iconfile = pathlib.Path(icondir) / 'hdots.png'
+                w.setStyleSheet(
+                    """
+                    QSplitter::handle { height: %spx; image: url(%s); }
+                    """ % (self.thumb_px, iconfile))
         self.widget = w
         w.setChildrenCollapsible(True)
 
@@ -1602,6 +1600,18 @@ class GridBox(ContainerBase):
         self.widget.layout().addWidget(w, row, col)
         self.make_callback('widget-added', child)
 
+    def insert_cell(self, row, col):
+        raise NotImplementedError("insert_cell needs to be implemented!")
+
+    def insert_row(self, index):
+        raise NotImplementedError("insert_row needs to be implemented!")
+
+    def append_row(self):
+        raise NotImplementedError("append_row needs to be implemented!")
+
+    def delete_row(self, index):
+        raise NotImplementedError("delete_row needs to be implemented!")
+
 
 class ToolbarAction(WidgetBase):
     def __init__(self):
@@ -1841,6 +1851,9 @@ class TopLevelMixin(object):
     def unmaximize(self):
         self.widget.showNormal()
 
+    def is_maximized(self):
+        return self.widget.isMaximized()
+
     def fullscreen(self):
         self.widget.showFullScreen()
 
@@ -1860,6 +1873,87 @@ class TopLevelMixin(object):
         self.widget.setWindowTitle(title)
 
 
+class MDIWindow(TopLevelMixin, WidgetBase):
+    def __init__(self, parent, child, title=''):
+        """NOTE: this widget is not meant to be instantiated except *inside*
+        of MDIWidget implementation.
+        """
+        WidgetBase.__init__(self)
+        self.parent = parent
+        w = QtGui.QMdiSubWindow(parent.get_widget())
+        # remove Qt logo from subwindow
+        w.setWindowIcon(QIcon(QPixmap(1, 1)))
+        self.widget = w
+
+        child_w = child.get_widget()
+        w.setWidget(child_w)
+
+        # NOTE: we fire the page-switch callback by intercepting the
+        # focus event on the subwindow, rather than off of the
+        # subWindowActivated signal because the latter fires if
+        # the widget accepts focus when the mouse enters the window,
+        # whereas this approach one actually has to click in the window
+        # or title bar.
+
+        def _focus_cb(event):
+            if event.gotFocus():
+                self.parent._cb_redirect(self.widget)
+
+        parent_w = parent.get_widget()
+        parent_w.addSubWindow(w)
+
+        w.focusInEvent = _focus_cb
+        # Monkey-patching the widget to take control of resize and move
+        # events
+        w._resizeEvent = w.resizeEvent
+        w.resizeEvent = lambda event: self._window_resized(event, w, child)
+        w._moveEvent = w.moveEvent
+        w.moveEvent = lambda event: self._window_moved(event, w, child)
+        w._closeEvent = w.closeEvent
+        w.closeEvent = lambda event: self._window_closed(event, w, child)
+        # attach title to child
+        child.extdata.tab_title = title
+
+        TopLevelMixin.__init__(self, title=title)
+
+        # does child have a previously saved size
+        size = child.extdata.get('mdi_size', None)
+        if size is not None:
+            wd, ht = size
+            w.resize(wd, ht)
+
+        # does child have a previously saved position
+        pos = child.extdata.get('mdi_pos', None)
+        if pos is not None:
+            x, y = pos
+            w.move(x, y)
+
+        child.show()
+        w.show()
+
+    def _window_resized(self, event, subwin, child):
+        qsize = event.size()
+        wd, ht = qsize.width(), qsize.height()
+        # save size into child widget metadata
+        child.extdata.mdi_size = (wd, ht)
+        subwin._resizeEvent(event)
+
+    def _window_moved(self, event, subwin, child):
+        qpos = event.pos()
+        x, y = qpos.x(), qpos.y()
+        # save position into child widget metadata
+        child.extdata.mdi_pos = (x, y)
+        subwin._moveEvent(event)
+
+    def _window_closed(self, event, subwin, widget):
+        # let the application deal with this if desired in page-close
+        # callback
+        event.ignore()
+        # self.widget.removeSubWindow(subwin)
+
+        self.make_callback('close')
+
+
 class TopLevel(TopLevelMixin, ContainerBase):
 
     def __init__(self, title=None):
@@ -2134,6 +2228,8 @@ def make_widget(title, wtype):
         w = ProgressBar()
     elif wtype == 'menubar':
         w = Menubar()
+    elif wtype == 'dial':
+        w = Dial()
     else:
         raise ValueError("Bad wtype=%s" % wtype)
     return w
