@@ -15,6 +15,12 @@ import logging.handlers
 import threading
 import traceback
 
+if sys.version_info < (3, 8):
+    # Python 3.7
+    from importlib_metadata import entry_points
+else:
+    from importlib.metadata import entry_points
+
 # Local application imports
 from ginga.misc.Bunch import Bunch
 from ginga.misc import Task, ModuleManager, Settings, log
@@ -98,6 +104,8 @@ plugins = [
           menu="Blink Channels [G]", category='Analysis', ptype='global'),
     Bunch(module='Blink', workspace='dialogs', menu='Blink Images',
           category='Analysis', ptype='local'),
+    Bunch(module='Crosshair', workspace='left', category='Analysis',
+          ptype='local'),
     Bunch(module='Cuts', workspace='dialogs', category='Analysis',
           ptype='local'),
     Bunch(module='LineProfile', workspace='dialogs',
@@ -136,10 +144,10 @@ plugins = [
           ptype='local'),
     Bunch(module='Catalogs', workspace='dialogs', category='Utils',
           ptype='local'),
-    Bunch(module='Crosshair', workspace='dialogs', category='Utils',
-          ptype='local'),
     Bunch(module='Drawing', workspace='dialogs', category='Utils',
           ptype='local'),
+    ## Bunch(module='Pipeline', workspace='dialogs', category='Utils',
+    ##       ptype='local'),
     Bunch(module='FBrowser', workspace='dialogs', category='Utils',
           ptype='local'),
     Bunch(module='ChangeHistory', tab='History', workspace='right',
@@ -197,12 +205,12 @@ class ReferenceViewer(object):
                 self.add_plugin_spec(spec)
 
     def add_separately_distributed_plugins(self):
-        from pkg_resources import iter_entry_points
         groups = ['ginga.rv.plugins']
         available_methods = []
 
         for group in groups:
-            for entry_point in iter_entry_points(group=group, name=None):
+            discovered_plugins = entry_points().get(group, [])
+            for entry_point in discovered_plugins:
                 try:
                     method = entry_point.load()
                     available_methods.append(method)
@@ -267,9 +275,6 @@ class ReferenceViewer(object):
         add_argument("--opencv", dest="opencv", default=False,
                      action="store_true",
                      help="Use OpenCv acceleration")
-        add_argument("--opencl", dest="opencl", default=False,
-                     action="store_true",
-                     help="Use OpenCL acceleration")
         add_argument("--opengl", dest="opengl", default=False,
                      action="store_true",
                      help="Use OpenGL acceleration")
@@ -416,7 +421,7 @@ class ReferenceViewer(object):
                 assert wcsmod.use(wcspkg) is True
         except Exception as e:
             logger.warning(
-                "failed to set WCS package preference: %s" % (str(e)))
+                "failed to set WCS package preference '{}': {}".format(wcspkg, e))
 
         # User wants to customize the FITS package?
         if options.fitspkg:
@@ -435,27 +440,12 @@ class ReferenceViewer(object):
 
         except Exception as e:
             logger.warning(
-                "failed to set FITS package preference: %s" % (str(e)))
+                "failed to set FITS package preference '{}': {}".format(fitspkg, e))
 
-        # Check whether user wants to use OpenCv
-        use_opencv = settings.get('use_opencv', False)
-        if use_opencv or options.opencv:
-            from ginga import trcalc
-            try:
-                trcalc.use('opencv')
-            except Exception as e:
-                logger.warning(
-                    "failed to set OpenCv preference: %s" % (str(e)))
-
-        # Check whether user wants to use OpenCL
-        use_opencl = settings.get('use_opencl', False)
-        if use_opencl or options.opencl:
-            from ginga import trcalc
-            try:
-                trcalc.use('opencl')
-            except Exception as e:
-                logger.warning(
-                    "failed to set OpenCL preference: %s" % (str(e)))
+        # Check whether user specified deprecated --opencv option
+        if options.opencv:
+            logger.warning("--opencv switch is deprecated; "
+                           "OpenCv will be used by default if installed")
 
         if options.opengl:
             settings.set(use_opengl=True)
@@ -693,8 +683,7 @@ class ReferenceViewer(object):
                 # if there is a network component, start it
                 if hasattr(ginga_shell, 'start'):
                     logger.info("starting network interface...")
-                    task = Task.FuncTask2(ginga_shell.start)
-                    thread_pool.addTask(task)
+                    ginga_shell.start()
 
                 # Main loop to handle GUI events
                 logger.info("entering mainloop...")
