@@ -1,7 +1,6 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
-"""
-Perform quick astronomical stellar analysis.
+"""Perform quick astronomical stellar analysis.
 
 **Plugin Type: Local**
 
@@ -77,7 +76,7 @@ also be marked with a point centered on the object and additionally the
 pan position will be set to the found center.
 
 .. figure:: figures/pick-contour.png
-   :width: 400px
+   :width: 300px
    :align: center
    :alt: Contour tab of Pick area
 
@@ -97,10 +96,11 @@ scroll wheel (mouse button 2) to set the pan position in the plot.
    "FWHM" tab of ``Pick`` area.
 
 The "FWHM" tab will show a FWHM plot.
-The blue lines show measurements in the X direction and the green lines
+The purple lines show measurements in the X direction and the green lines
 show measurements in the Y direction.  The solid lines indicate actual
 pixel values and the dotted lines indicate the fitted 1D function.
-The shaded green and blue regions indicate the FWHM measurements.
+The shaded purple and green regions indicate the FWHM measurements for the
+respective axes.
 
 .. figure:: figures/pick-radial.png
    :width: 400px
@@ -110,20 +110,29 @@ The shaded green and blue regions indicate the FWHM measurements.
    "Radial" tab of ``Pick`` area.
 
 The "Radial" tab contains a radial profile plot.
-Plotted points in blue are data values, and a line is fitted to the
+Plotted points in purple are data values, and a line is fitted to the
 data.
 
-.. figure:: figures/pick-cuts.png
-   :width: 800px
+.. figure:: figures/pick-ee.png
+   :width: 600px
    :align: center
-   :alt: Cut tab of Pick area
+   :alt: EE tab of Pick area
 
-   "Cut" tab of ``Pick`` area.
+   "EE" tab of ``Pick`` area.
 
-The "Cuts" tab contains a profile plot for the vertical and horizontal
-cuts represented by the crosshairs present in "Quick Mode" ON.  This plot
-is updated in real time as the pick area is moved.  In "Quick Mode" OFF,
-this plot is not updated.
+The "EE" tab contains a plot of fractional encircled and ensquared energies
+(EE) in purple and green, respectively, for the chosen target. Simple
+background subtraction is done in a way that is consistent with FWHM
+calculations before EE values are measured. The sampling and total radii,
+shown as black dashed lines, can be set in the "Settings" tab; when these are
+changed, click "Redo Pick" to update the plot and measurements.
+The measured EE values at the given sampling radius are also displayed in the
+"Readout" tab. When reporting is requested, the EE values at the given sampling
+radius and the radius itself will be recorded under "Report" table, along with
+other information.
+
+When "Show Candidates" is active, the candidates near the edges of the bounding
+box will not have EE values (set to 0).
 
 .. figure:: figures/pick-readout.png
    :width: 400px
@@ -144,7 +153,7 @@ There are two buttons and three check boxes in this tab:
 * The "From Peak" check box changes the behavior of "Quick Mode" slightly
   as described above.
 * If "Center on pick" is checked, the shape will be recentered on the
-  located center, if found (i.e. the shape "tracks" the pick)
+  located center, if found (i.e., the shape "tracks" the pick).
 
 .. figure:: figures/pick-controls.png
    :width: 400px
@@ -184,7 +193,7 @@ candidate is added to the table.  If the "Record Picks automatically"
 checkbox is checked, then any candidates are added to the table
 automatically.
 
-.. note:: If the "Show candidates" checkbox in the "Settings" tab is
+.. note:: If the "Show Candidates" checkbox in the "Settings" tab is
           checked, then *all* objects found in the region (according to
           the settings) will be added to the table instead of just the
           selected candidate.
@@ -211,8 +220,7 @@ The image cutout will be taken from this central area and so the "Image"
 tab will still have content.  It will also be marked with a central red
 "X".
 
-The contour plot will still be produced from the cutout, and the cuts
-plot will be updated in "Quick Mode".
+The contour plot will still be produced from the cutout.
 
 .. figure:: figures/pick-contour-no-candidate.png
    :width: 400px
@@ -234,7 +242,7 @@ All the other plots will be cleared.
 
 The "Settings" tab controls aspects of the search within the pick area:
 
-* The "Show candidates" checkbox controls whether all detected sources
+* The "Show Candidates" checkbox controls whether all detected sources
   are marked or not (as shown in the figure below).  Additionally, if
   checked, then all the found objects are added to the pick log table
   when using the "Report" controls.
@@ -260,20 +268,27 @@ The "Settings" tab controls aspects of the search within the pick area:
 * The "Calc center" parameter is used to determine whether the center
   is calculated from FWHM fitting ("fwhm") or centroiding ("centroid").
 * The "FWHM fitting" parameter is used to determine which function is
-  is used for FWHM fitting ("gaussian" or "moffat").
+  is used for FWHM fitting ("gaussian" or "moffat"). The option to use
+  "lorentz" is also available if "calc_fwhm_lib" is set to "astropy"
+  in ``~/.ginga/plugin_Pick.cfg``.
 * The "Contour Interpolation" parameter is used to set the interpolation
   method used in rendering the background image in the "Contour" plot.
+* The "EE total radius" defines the radius (for encircled energy) and box
+  half-width (for ensquared energy) in pixels where EE fraction is expected to
+  be 1 (i.e., all the flux for a point-spread function is contained within).
+* The "EE sampling radius" is the radius in pixel used to sample the measured
+  EE curves for reporting.
 
-The "Redo Pick" button will redo the search operation.  It's convenient
+The "Redo Pick" button will redo the search operation.  It is convenient
 if you have changed some parameters and want to see the effect based on the
 current pick area without disturbing it.
 
 .. figure:: figures/pick-candidates.png
-   :width: 800px
+   :width: 600px
    :align: center
-   :alt: The channel viewer when "Show candidates" is checked.
+   :alt: The channel viewer when "Show Candidates" is checked.
 
-   The channel viewer when "Show candidates" is checked.
+   The channel viewer when "Show Candidates" is checked.
 
 **User Configuration**
 
@@ -288,8 +303,8 @@ import numpy as np
 
 from ginga.gw import Widgets, Viewers
 from ginga.misc import Bunch
-from ginga.util import iqcalc, wcs, contour
-from ginga import GingaPlugin, colors, cmap, trcalc
+from ginga.util import wcs, contour
+from ginga import GingaPlugin, cmap, trcalc
 
 try:
     from ginga.gw import Plot
@@ -321,7 +336,6 @@ class Pick(GingaPlugin.LocalPlugin):
         self.contour_plot = None
         self.fwhm_plot = None
         self.radial_plot = None
-        self.cuts_plot = None
         self.contour_interp_methods = trcalc.interpolation_methods
 
         # types of pick shapes that can be drawn
@@ -348,8 +362,6 @@ class Pick(GingaPlugin.LocalPlugin):
         self.lock = threading.RLock()
         self.lock2 = threading.RLock()
         self.ev_intr = threading.Event()
-        self.tmr_quick = fv.get_timer()
-        self.tmr_quick.set_callback('expired', self.quick_timer_cb)
         self._wd, self._ht = 400, 300
         self._split_sizes = [self._ht, self._ht]
 
@@ -362,6 +374,16 @@ class Pick(GingaPlugin.LocalPlugin):
         self.rpt_wrt_interval = self.settings.get('report_write_interval',
                                                   30.0)
 
+        if self.iqcalc_lib == 'astropy':
+            self.logger.debug('Using iqcalc_astropy')
+            from ginga.util import iqcalc_astropy as iqcalc
+        else:  # Falls back to native
+            self.logger.debug('Using native iqcalc')
+            from ginga.util import iqcalc
+
+        if not iqcalc.have_scipy:
+            raise ImportError('Please install scipy to use this plugin')
+
         self.iqcalc = iqcalc.IQCalc(self.logger)
         self.copy_attrs = ['transforms', 'cutlevels']
         if (self.settings.get('pick_cmap_name', None) is None and
@@ -377,26 +399,12 @@ class Pick(GingaPlugin.LocalPlugin):
         canvas.set_callback('draw-event', self.draw_cb)
         canvas.set_callback('edit-event', self.edit_cb)
         canvas.add_draw_mode('move', down=self.btn_down,
-                             move=self.btn_drag, up=self.btn_up,
-                             hover=self.hover_cb)
+                             move=self.btn_drag, up=self.btn_up)
         canvas.register_for_cursor_drawing(self.fitsimage)
         canvas.set_surface(self.fitsimage)
         canvas.set_draw_mode('move')
         self.canvas = canvas
 
-        # quick cross marker
-        # these colors form the crosshair marking the cuts in canvas
-        # and also the cuts plot
-        cname1 = self.settings.get('quick_h_cross_color', 'magenta2')
-        color1 = colors.lookup_color(cname1, format='hash')
-        cname2 = self.settings.get('quick_v_cross_color', 'sienna3')
-        color2 = colors.lookup_color(cname2, format='hash')
-        self.cross = self.dc.CompoundObject(
-            self.dc.Line(0, 10, 20, 10, color=color1),
-            self.dc.Line(10, 0, 10, 20, color=color2))
-        if self.quick_mode:
-            self.canvas.add(self.cross)
-
         self.have_mpl = have_mpl
         self.gui_up = False
 
@@ -408,16 +416,15 @@ class Pick(GingaPlugin.LocalPlugin):
             self.pickshape = 'box'
         self.candidate_color = self.settings.get('color_candidate', 'orange')
         self.quick_mode = self.settings.get('quick_mode', False)
-        self.quick_update_interval = self.settings.get('quick_update_interval',
-                                                       0.25)
         self.from_peak = self.settings.get('quick_from_peak', True)
-        self.drag_only = self.settings.get('quick_drag_only', True)
 
         # Peak finding parameters and selection criteria
         self.max_side = self.settings.get('max_side', 1024)
         self.radius = self.settings.get('radius', 10)
+        self.ee_total_radius = self.settings.get('ee_total_radius', 10.0)
+        self.ee_sampling_radius = self.settings.get('ee_sampling_radius', 2.5)
         self.threshold = self.settings.get('threshold', None)
-        self.min_fwhm = self.settings.get('min_fwhm', 2.0)
+        self.min_fwhm = self.settings.get('min_fwhm', 1.5)
         self.max_fwhm = self.settings.get('max_fwhm', 50.0)
         self.min_ellipse = self.settings.get('min_ellipse', 0.5)
         self.edgew = self.settings.get('edge_width', 0.01)
@@ -429,6 +436,9 @@ class Pick(GingaPlugin.LocalPlugin):
         self.center_algs = ['fwhm', 'centroid']
         self.center_alg = self.settings.get('calc_center_alg', 'fwhm')
         self.fwhm_algs = ['gaussian', 'moffat']
+        self.iqcalc_lib = self.settings.get('calc_fwhm_lib', 'native')
+        if self.iqcalc_lib == 'astropy':
+            self.fwhm_algs.append('lorentz')
         self.fwhm_alg = self.settings.get('calc_fwhm_alg', 'gaussian')
         self.center_on_pick = self.settings.get('center_on_pick', False)
 
@@ -442,6 +452,8 @@ class Pick(GingaPlugin.LocalPlugin):
         columns = [("RA", 'ra_txt'), ("DEC", 'dec_txt'), ("Equinox", 'equinox'),
                    ("X", 'x'), ("Y", 'y'), ("FWHM", 'fwhm'),
                    ("FWHM_X", 'fwhm_x'), ("FWHM_Y", 'fwhm_y'),
+                   ("EE_circ", 'encircled_energy'), ("EE_sq", 'ensquared_energy'),
+                   ("EE_r", 'ee_sampling_radius'),
                    ("Star Size", 'starsize'),
                    ("Ellip", 'ellipse'), ("Background", 'background'),
                    ("Sky Level", 'skylevel'), ("Brightness", 'brightness'),
@@ -458,13 +470,11 @@ class Pick(GingaPlugin.LocalPlugin):
                                                        'nearest')
 
     def build_gui(self, container):
-        assert iqcalc.have_scipy is True, \
-            Exception("Please install python-scipy to use this plugin")
-
         vtop = Widgets.VBox()
         vtop.set_border_width(4)
 
-        box, sw, orientation = Widgets.get_oriented_box(container)
+        box, sw, orientation = Widgets.get_oriented_box(container,
+                                                        orientation=self.settings.get('orientation', None))
         box.set_border_width(4)
         box.set_spacing(2)
 
@@ -477,14 +487,14 @@ class Pick(GingaPlugin.LocalPlugin):
 
         cm, im = self.fv.cm, self.fv.im
 
+        # Set up "Image" tab viewer
         di = Viewers.CanvasView(logger=self.logger)
         width, height = self._wd, self._ht
         di.set_desired_size(width, height)
-        di.enable_autozoom('off')
+        di.enable_autozoom('override')
         di.enable_autocuts('off')
         di.set_zoom_algorithm('rate')
         di.set_zoomrate(1.6)
-        di.zoom_to(2)
         settings = di.get_settings()
         settings.get_setting('zoomlevel').add_callback('set', self.zoomset, di)
 
@@ -522,16 +532,16 @@ class Pick(GingaPlugin.LocalPlugin):
         iw.resize(width, height)
         nb.add_widget(iw, title="Image")
 
+        # Set up "Contour" tab viewer
         if contour.have_skimage:
             # Contour plot, Ginga-style
             ci = Viewers.CanvasView(logger=self.logger)
             width, height = 400, 300
             ci.set_desired_size(width, height)
-            ci.enable_autozoom('once')
+            ci.enable_autozoom('override')
             ci.enable_autocuts('override')
             ci.set_zoom_algorithm('rate')
             ci.set_zoomrate(1.6)
-            ci.zoom_to(3)
             ci.set_autocut_params('histogram')
 
             t_ = ci.get_settings()
@@ -571,15 +581,10 @@ class Pick(GingaPlugin.LocalPlugin):
                 # Contour plot
                 self.contour_plot = plots.ContourPlot(
                     logger=self.logger, width=width, height=height)
-                if plots.MPL_GE_2_0:
-                    kwargs = {'facecolor': 'black'}
-                else:
-                    kwargs = {'axisbg': 'black'}
-                self.contour_plot.add_axis(**kwargs)
-                self.contour_plot.enable(pan=True, zoom=True)
+                self.contour_plot.add_axis(facecolor='black')
                 pw = Plot.PlotWidget(self.contour_plot)
                 pw.resize(width, height)
-                self.contour_plot.connect_ui()
+                self.contour_plot.enable(pan=True, zoom=True)
 
                 self.contour_interp_methods = ('bilinear', 'nearest', 'bicubic')
                 if self.contour_interpolation not in self.contour_interp_methods:
@@ -591,11 +596,7 @@ class Pick(GingaPlugin.LocalPlugin):
             # FWHM gaussians plot
             self.fwhm_plot = plots.FWHMPlot(logger=self.logger,
                                             width=width, height=height)
-            if plots.MPL_GE_2_0:
-                kwargs = {'facecolor': 'white'}
-            else:
-                kwargs = {'axisbg': 'white'}
-            self.fwhm_plot.add_axis(**kwargs)
+            self.fwhm_plot.add_axis(facecolor='white')
             pw = Plot.PlotWidget(self.fwhm_plot)
             pw.resize(width, height)
             nb.add_widget(pw, title="FWHM")
@@ -603,19 +604,18 @@ class Pick(GingaPlugin.LocalPlugin):
             # Radial profile plot
             self.radial_plot = plots.RadialPlot(logger=self.logger,
                                                 width=width, height=height)
-            self.radial_plot.add_axis(**kwargs)
+            self.radial_plot.add_axis(facecolor='white')
             pw = Plot.PlotWidget(self.radial_plot)
             pw.resize(width, height)
             nb.add_widget(pw, title="Radial")
 
-            # Cuts profile plot
-            self.cuts_plot = plots.CutsPlot(logger=self.logger,
-                                            width=width, height=height)
-            pw = Plot.PlotWidget(self.cuts_plot)
+            # EE profile plot
+            self.ee_plot = plots.EEPlot(logger=self.logger,
+                                        width=width, height=height)
+            self.ee_plot.add_axis(facecolor='white')
+            pw = Plot.PlotWidget(self.ee_plot)
             pw.resize(width, height)
-            ax = self.cuts_plot.add_axis()
-            ax.grid(True)
-            nb.add_widget(pw, title="Cuts")
+            nb.add_widget(pw, title="EE")
 
         fr = Widgets.Frame(self._textlabel)
 
@@ -636,6 +636,8 @@ class Pick(GingaPlugin.LocalPlugin):
                      'FWHM Y:', 'label', 'FWHM Y', 'llabel'),
                     ('FWHM:', 'label', 'FWHM', 'llabel',
                      'Star Size:', 'label', 'Star Size', 'llabel'),
+                    ('EE (circ):', 'label', 'Encircled energy', 'llabel',
+                     'EE (sq):', 'label', 'Ensquared energy', 'llabel'),
                     ('Sample Area:', 'label', 'Sample Area', 'llabel',
                      'Default Region', 'button', 'Pan to pick', 'button'),
                     ('Quick Mode', 'checkbutton', 'From Peak', 'checkbutton',
@@ -646,6 +648,8 @@ class Pick(GingaPlugin.LocalPlugin):
         self.w.update(b)
         b.zoom.set_text(self.fv.scale2text(di.get_scale()))
         self.wdetail = b
+        b.encircled_energy.set_tooltip("Encircled energy")
+        b.ensquared_energy.set_tooltip("Ensquared energy")
         b.default_region.add_callback('activated',
                                       lambda w: self.reset_region())
         b.default_region.set_tooltip("Reset region size to default")
@@ -734,6 +738,10 @@ class Pick(GingaPlugin.LocalPlugin):
                      "FWHM fitting", 'combobox'),
                     ('Contour Interpolation:', 'label', 'xlbl_cinterp', 'label',
                      'Contour Interpolation', 'combobox'),
+                    ('EE total radius:', 'label', 'xlbl_ee_total_radius', 'label',
+                     'EE total radius', 'spinfloat'),
+                    ('EE sampling radius:', 'label', 'xlbl_ee_radius', 'label',
+                     'EE sampling radius', 'spinfloat')
                     )
 
         w, b = Widgets.build_info(captions, orientation=orientation)
@@ -750,6 +758,8 @@ class Pick(GingaPlugin.LocalPlugin):
         b.calc_center.set_tooltip("How to calculate the center of object")
         b.fwhm_fitting.set_tooltip("Function for fitting the FWHM")
         b.contour_interpolation.set_tooltip("Interpolation for use in contour plot")
+        b.ee_total_radius.set_tooltip("Radius where EE fraction is 1")
+        b.ee_sampling_radius.set_tooltip("Radius for EE sampling")
 
         def chg_pickshape(w, idx):
             pickshape = self.drawtypes[idx]
@@ -776,6 +786,28 @@ class Pick(GingaPlugin.LocalPlugin):
         b.xlbl_radius.set_text(str(self.radius))
         b.radius.add_callback('value-changed', chg_radius)
 
+        # EE total radius control
+        b.ee_total_radius.set_limits(0.1, 200.0, incr_value=0.1)
+        b.ee_total_radius.set_value(self.ee_total_radius)
+
+        def chg_ee_total_radius(w, val):
+            self.ee_total_radius = float(val)
+            self.w.xlbl_ee_total_radius.set_text(str(self.ee_total_radius))
+            return True
+        b.xlbl_ee_total_radius.set_text(str(self.ee_total_radius))
+        b.ee_total_radius.add_callback('value-changed', chg_ee_total_radius)
+
+        # EE sampling radius control
+        b.ee_sampling_radius.set_limits(0.1, 200.0, incr_value=0.1)
+        b.ee_sampling_radius.set_value(self.ee_sampling_radius)
+
+        def chg_ee_sampling_radius(w, val):
+            self.ee_sampling_radius = float(val)
+            self.w.xlbl_ee_radius.set_text(str(self.ee_sampling_radius))
+            return True
+        b.xlbl_ee_radius.set_text(str(self.ee_sampling_radius))
+        b.ee_sampling_radius.add_callback('value-changed', chg_ee_sampling_radius)
+
         # threshold control
         def chg_threshold(w):
             threshold = None
@@ -1161,32 +1193,33 @@ class Pick(GingaPlugin.LocalPlugin):
         try:
             if self.contour_image is not None:
                 cv = self.contour_image
-                cv.set_data(data)
-                # copy orientation of main image, so that contour will
-                # make sense.  Don't do rotation, for now.
-                flips = self.fitsimage.get_transforms()
-                cv.transform(*flips)
-                #rot_deg = self.fitsimage.get_rotation()
-                #cv.rotate(rot_deg)
+                with cv.suppress_redraw:
+                    cv.set_data(data)
+                    # copy orientation of main image, so that contour will
+                    # make sense.  Don't do rotation, for now.
+                    flips = self.fitsimage.get_transforms()
+                    cv.transform(*flips)
+                    #rot_deg = self.fitsimage.get_rotation()
+                    #cv.rotate(rot_deg)
 
-                cv.panset_xy(x, y)
+                    cv.panset_xy(x, y)
 
-                canvas = self.contour_canvas
-                try:
-                    canvas.delete_object_by_tag('_$cntr', redraw=False)
-                except KeyError:
-                    pass
+                    canvas = self.contour_canvas
+                    try:
+                        canvas.delete_object_by_tag('_$cntr', redraw=False)
+                    except KeyError:
+                        pass
 
-                # calculate contour polygons
-                contour_grps = contour.calc_contours(data, self.num_contours)
+                    # calculate contour polygons
+                    contour_grps = contour.calc_contours(data, self.num_contours)
 
-                # get compound polygons object
-                c_obj = contour.create_contours_obj(canvas, contour_grps,
-                                                    colors=['black'],
-                                                    linewidth=2)
-                canvas.add(c_obj, tag='_$cntr')
+                    # get compound polygons object
+                    c_obj = contour.create_contours_obj(canvas, contour_grps,
+                                                        colors=['black'],
+                                                        linewidth=2)
+                    canvas.add(c_obj, tag='_$cntr')
 
-            if self.contour_plot is not None:
+            elif self.contour_plot is not None:
                 self.contour_plot.plot_contours_data(
                     x, y, data, num_contours=self.num_contours)
 
@@ -1197,18 +1230,18 @@ class Pick(GingaPlugin.LocalPlugin):
     def clear_contours(self):
         if self.contour_image is not None:
             self.contour_canvas.delete_all_objects()
-        if self.contour_plot is not None:
+        elif self.contour_plot is not None:
             self.contour_plot.clear()
 
     def plot_fwhm(self, qs, image):
         # Make a FWHM plot
-        x, y, radius = qs.x, qs.y, qs.fwhm_radius
+        x, y = qs.x - self.pick_x1, qs.y - self.pick_y1
+        radius = qs.fwhm_radius
 
         try:
-            self.fwhm_plot.plot_fwhm(x, y, radius, image,
-                                     cutout_data=self.pick_data,
-                                     iqcalc=self.iqcalc,
-                                     fwhm_method=self.fwhm_alg)
+            self.fwhm_plot.plot_fwhm_data(x, y, radius, self.pick_data,
+                                          iqcalc=self.iqcalc,
+                                          fwhm_method=self.fwhm_alg)
 
         except Exception as e:
             self.logger.error("Error making fwhm plot: %s" % (
@@ -1231,6 +1264,20 @@ class Pick(GingaPlugin.LocalPlugin):
     def clear_radial(self):
         self.radial_plot.clear()
 
+    def plot_ee(self, qs):
+        # Make a EE plot
+        try:
+            self.ee_plot.plot_ee(
+                encircled_energy_function=qs.encircled_energy_fn,
+                ensquared_energy_function=qs.ensquared_energy_fn,
+                sampling_radius=self.ee_sampling_radius,
+                total_radius=self.ee_total_radius)
+        except Exception as e:
+            self.logger.error("Error making EE plot: %s" % (str(e)))
+
+    def clear_ee(self):
+        self.ee_plot.clear()
+
     def close(self):
         self.fv.stop_local_plugin(self.chname, str(self))
         return True
@@ -1372,11 +1419,13 @@ class Pick(GingaPlugin.LocalPlugin):
                 # Evaluate those peaks
                 self.update_status("Evaluating %d bright peaks..." % (
                     num_peaks))
-                objlist = self.iqcalc.evaluate_peaks(peaks, data,
-                                                     fwhm_radius=self.radius,
-                                                     cb_fn=cb_fn,
-                                                     ev_intr=self.ev_intr,
-                                                     fwhm_method=self.fwhm_alg)
+                objlist = self.iqcalc.evaluate_peaks(
+                    peaks, data,
+                    fwhm_radius=self.radius,
+                    cb_fn=cb_fn,
+                    ev_intr=self.ev_intr,
+                    fwhm_method=self.fwhm_alg,
+                    ee_total_radius=self.ee_total_radius)
 
                 num_candidates = len(objlist)
                 if num_candidates == 0:
@@ -1416,9 +1465,6 @@ class Pick(GingaPlugin.LocalPlugin):
             self.fv.gui_do(self.update_pick, serialnum, results, qs,
                            x1, y1, wd, ht, data, pickobj, msg)
 
-    def _make_report_header(self):
-        return self.rpt_header + '\n'
-
     def _make_report(self, vip_img, qs):
         d = Bunch.Bunch()
         try:
@@ -1455,6 +1501,15 @@ class Pick(GingaPlugin.LocalPlugin):
             rpt_x = x + self.pixel_coords_offset
             rpt_y = y + self.pixel_coords_offset
 
+            # EE at sampling radius
+            try:
+                ee_circ = qs.encircled_energy_fn(self.ee_sampling_radius)
+                ee_sq = qs.ensquared_energy_fn(self.ee_sampling_radius)
+            except Exception as e:
+                self.logger.warning("Couldn't calculate EE at %.2f: %s" % (self.ee_sampling_radius, str(e)))
+                ee_circ = 0
+                ee_sq = 0
+
             # make a report in the form of a dictionary
             d.setvals(x=rpt_x, y=rpt_y,
                       ra_deg=ra_deg, dec_deg=dec_deg,
@@ -1464,6 +1519,8 @@ class Pick(GingaPlugin.LocalPlugin):
                       fwhm_x=qs.fwhm_x, fwhm_y=qs.fwhm_y,
                       ellipse=qs.elipse, background=qs.background,
                       skylevel=qs.skylevel, brightness=qs.brightness,
+                      encircled_energy=ee_circ, ensquared_energy=ee_sq,
+                      ee_sampling_radius=self.ee_sampling_radius,
                       starsize=starsize,
                       time_local=time.strftime("%Y-%m-%d %H:%M:%S",
                                                time.localtime()),
@@ -1533,11 +1590,13 @@ class Pick(GingaPlugin.LocalPlugin):
             self.wdetail.fwhm_x.set_text('%.3f' % fwhm_x)
             self.wdetail.fwhm_y.set_text('%.3f' % fwhm_y)
             self.wdetail.fwhm.set_text('%.3f' % fwhm)
-            self.wdetail.object_x.set_text('%.3f' % (d.x))
-            self.wdetail.object_y.set_text('%.3f' % (d.y))
+            self.wdetail.object_x.set_text('%.3f' % d.x)
+            self.wdetail.object_y.set_text('%.3f' % d.y)
             self.wdetail.sky_level.set_text('%.3f' % qs.skylevel)
             self.wdetail.background.set_text('%.3f' % qs.background)
             self.wdetail.brightness.set_text('%.3f' % qs.brightness)
+            self.wdetail.encircled_energy.set_text('%.3f' % d.encircled_energy)
+            self.wdetail.ensquared_energy.set_text('%.3f' % d.ensquared_energy)
             self.wdetail.ra.set_text(d.ra_txt)
             self.wdetail.dec.set_text(d.dec_txt)
             self.wdetail.equinox.set_text(str(d.equinox))
@@ -1568,6 +1627,7 @@ class Pick(GingaPlugin.LocalPlugin):
                 self.plot_contours(vip_img)
                 self.plot_fwhm(qs, vip_img)
                 self.plot_radial(qs, vip_img)
+                self.plot_ee(qs)
 
         except Exception as e:
             errmsg = "Error calculating quality metrics: %s" % (
@@ -1583,7 +1643,8 @@ class Pick(GingaPlugin.LocalPlugin):
             #self.update_status("Error")
             for key in ('sky_level', 'background', 'brightness',
                         'star_size', 'fwhm_x', 'fwhm_y',
-                        'ra', 'dec', 'object_x', 'object_y'):
+                        'ra', 'dec', 'object_x', 'object_y',
+                        'encircled_energy', 'ensquared_energy'):
                 self.wdetail[key].set_text('')
             self.wdetail.fwhm.set_text('Failed')
             self.w.btn_bg_cut.set_enabled(False)
@@ -1600,6 +1661,7 @@ class Pick(GingaPlugin.LocalPlugin):
                 # TODO: could calc background based on numpy calc
                 self.clear_fwhm()
                 self.clear_radial()
+                self.clear_ee()
 
         self.w.btn_intr_eval.set_enabled(False)
         self.pickimage.redraw(whence=3)
@@ -1614,8 +1676,6 @@ class Pick(GingaPlugin.LocalPlugin):
     def redo_quick(self):
         vip_img = self.fitsimage.get_vip()
 
-        self.cuts_plot.clear()
-
         obj = self.pick_obj
         if obj is None:
             return
@@ -1625,35 +1685,6 @@ class Pick(GingaPlugin.LocalPlugin):
         self.pick_x1, self.pick_y1 = x1, y1
         self.pick_data = data
 
-        #ht, wd = data.shape[:2]
-        #x, y = wd // 2, ht // 2
-        x, y = shape.get_center_pt()[:2]
-        x, y = x - x1, y - y1
-        radius = min(x, y)
-
-        # update cross lines
-        hl = self.cross.objects[0]
-        hl.x1, hl.y1, hl.x2, hl.y2 = (x1 + x - radius, y1 + y,
-                                      x1 + x + radius, y1 + y)
-        vl = self.cross.objects[1]
-        vl.x1, vl.y1, vl.x2, vl.y2 = (x1 + x, y1 + y - radius,
-                                      x1 + x, y1 + y + radius)
-
-        # Get points on the lines
-        x0, y0, xarr, yarr = self.iqcalc.cut_cross(x, y, radius, data)
-
-        # plot horizontal cut
-        xpts = np.arange(len(xarr))
-        self.cuts_plot.plot(xpts, xarr, color=hl.color,
-                            xtitle="Line Index", ytitle="Pixel Value",
-                            title=None, rtitle="Cuts",
-                            alpha=1.0, linewidth=1.0, linestyle='-')
-
-        # plot vertical cut
-        ypts = np.arange(len(yarr))
-        self.cuts_plot.plot(ypts, yarr, color=vl.color,
-                            alpha=1.0, linewidth=1.0, linestyle='-')
-
     def calc_quick(self):
         if self.pick_data is None:
             return
@@ -1999,31 +2030,6 @@ class Pick(GingaPlugin.LocalPlugin):
 
         return True
 
-    def hover_cb(self, canvas, event, data_x, data_y, viewer):
-
-        if self.quick_mode and (not self.drag_only):
-            if self.pick_obj is None:
-                return False
-            shape = self.pick_obj.objects[0]
-            shape.color = 'cyan'
-            shape.linestyle = 'dash'
-            point = self.pick_obj.objects[1]
-            point.color = 'red'
-
-            shape.move_to(data_x, data_y)
-            self.canvas.update_canvas()
-
-            self.redo_quick()
-            # set timer
-            self.tmr_quick.set(self.quick_update_interval)
-
-            return True
-
-        return False
-
-    def quick_timer_cb(self, timer):
-        self.calc_quick()
-
     def set_mode_cb(self, mode, tf):
         if tf:
             self.canvas.set_draw_mode(mode)
@@ -2033,11 +2039,6 @@ class Pick(GingaPlugin.LocalPlugin):
 
     def quick_mode_cb(self, w, tf):
         self.quick_mode = tf
-        if self.quick_mode:
-            self.canvas.add(self.cross)
-            self.redo_quick()
-        else:
-            self.canvas.delete_object(self.cross)
         return True
 
     def from_peak_cb(self, w, tf):
