@@ -334,6 +334,12 @@ class Preferences(GingaPlugin.LocalPlugin):
         self.imap_names = imap.get_names()
         self.zoomalg_names = ('step', 'rate')
 
+        # get Preferences preferences
+        prefs = self.fv.get_preferences()
+        self.settings = prefs.create_category('plugin_Preferences')
+        self.settings.add_defaults(orientation=None)
+        self.settings.load(onError='silent')
+
         self.t_ = self.fitsimage.get_settings()
         self.autocuts_cache = {}
         self.gui_up = False
@@ -357,7 +363,7 @@ class Preferences(GingaPlugin.LocalPlugin):
             'set', self.autocenter_changed_ext_cb)
         self.t_.get_setting('autocuts').add_callback(
             'set', self.autocuts_changed_ext_cb)
-        for key in ['switchnew', 'raisenew', 'genthumb']:
+        for key in ['switchnew', 'raisenew', 'genthumb', 'auto_orient']:
             self.t_.get_setting(key).add_callback(
                 'set', self.set_chprefs_ext_cb)
 
@@ -406,7 +412,8 @@ class Preferences(GingaPlugin.LocalPlugin):
         top = Widgets.VBox()
         top.set_border_width(4)
 
-        vbox, sw, orientation = Widgets.get_oriented_box(container)
+        vbox, sw, orientation = Widgets.get_oriented_box(container,
+                                                         orientation=self.settings.get('orientation', None))
         self.orientation = orientation
         #vbox.set_border_width(4)
         vbox.set_spacing(2)
@@ -667,7 +674,7 @@ class Preferences(GingaPlugin.LocalPlugin):
         b.stretch_factor.set_enabled(zoomalg != 'step')
 
         zoomrate = self.t_.get('zoom_rate', math.sqrt(2.0))
-        b.zoom_rate.set_limits(1.1, 3.0, incr_value=0.1)
+        b.zoom_rate.set_limits(1.01, 10.0, incr_value=0.1)
         b.zoom_rate.set_value(zoomrate)
         b.zoom_rate.set_decimals(8)
         b.zoom_rate.set_enabled(zoomalg != 'step')
@@ -765,7 +772,8 @@ class Preferences(GingaPlugin.LocalPlugin):
                     ('Zoom New:', 'label', 'Zoom New', 'combobox'),
                     ('Center New:', 'label', 'Center New', 'combobox'),
                     ('Follow New', 'checkbutton', 'Raise New', 'checkbutton'),
-                    ('Create thumbnail', 'checkbutton'),
+                    ('Create thumbnail', 'checkbutton',
+                     'Auto Orient', 'checkbutton'),
                     )
         w, b = Widgets.build_info(captions, orientation=orientation)
         self.w.update(b)
@@ -810,6 +818,7 @@ class Preferences(GingaPlugin.LocalPlugin):
         b.follow_new.set_tooltip("View new images as they arrive")
         b.raise_new.set_tooltip("Raise and focus tab for new images")
         b.create_thumbnail.set_tooltip("Create thumbnail for new images")
+        b.auto_orient.set_tooltip("Auto orient new images")
 
         self.w.follow_new.set_state(True)
         self.w.follow_new.add_callback('activated', self.set_chprefs_cb)
@@ -817,6 +826,8 @@ class Preferences(GingaPlugin.LocalPlugin):
         self.w.raise_new.add_callback('activated', self.set_chprefs_cb)
         self.w.create_thumbnail.set_state(True)
         self.w.create_thumbnail.add_callback('activated', self.set_chprefs_cb)
+        self.w.auto_orient.set_state(True)
+        self.w.auto_orient.add_callback('activated', self.set_chprefs_cb)
 
         fr.set_widget(w)
         vbox.add_widget(fr, stretch=0)
@@ -1391,12 +1402,13 @@ class Preferences(GingaPlugin.LocalPlugin):
     def _update_pan_coords(self):
         pan_coord = self.t_.get('pan_coord', 'data')
         pan_x, pan_y = self.fitsimage.get_pan(coord=pan_coord)
-        #self.logger.debug("updating pan coords (%s) %f %f" % (pan_coord, pan_x, pan_y))
         if pan_coord == 'wcs':
             use_sex = self.w.wcs_sexagesimal.get_state()
             if use_sex:
-                pan_x = wcs.raDegToString(pan_x, format='%02d:%02d:%010.7f')
-                pan_y = wcs.decDegToString(pan_y, format='%s%02d:%02d:%09.7f')
+                pan_x = wcs.ra_deg_to_str(pan_x, precision=7,
+                                          format='%02d:%02d:%02d.%07d')
+                pan_y = wcs.dec_deg_to_str(pan_y, precision=7,
+                                           format='%s%02d:%02d:%02d.%07d')
         else:
             coord_offset = self.fv.settings.get('pixel_coords_offset', 0.0)
             pan_x += coord_offset
@@ -1431,14 +1443,16 @@ class Preferences(GingaPlugin.LocalPlugin):
         switchnew = (self.w.follow_new.get_state() != 0)
         raisenew = (self.w.raise_new.get_state() != 0)
         genthumb = (self.w.create_thumbnail.get_state() != 0)
+        auto_orient = (self.w.auto_orient.get_state() != 0)
         self.t_.set(switchnew=switchnew, raisenew=raisenew,
-                    genthumb=genthumb)
+                    genthumb=genthumb, auto_orient=auto_orient)
 
     def set_chprefs_ext_cb(self, *args):
         if self.gui_up:
             self.w.follow_new.set_state(self.t_['switchnew'])
             self.w.raise_new.set_state(self.t_['raisenew'])
             self.w.create_thumbnail.set_state(self.t_['genthumb'])
+            self.w.auto_orient.set_state(self.t_['auto_orient'])
 
     def set_profile_cb(self, *args):
         restore_scale = (self.w.restore_scale.get_state() != 0)
@@ -1576,6 +1590,8 @@ class Preferences(GingaPlugin.LocalPlugin):
         self.w.raise_new.set_state(prefs['raisenew'])
         prefs.setdefault('genthumb', True)
         self.w.create_thumbnail.set_state(prefs['genthumb'])
+        prefs.setdefault('auto_orient', True)
+        self.w.auto_orient.set_state(prefs['auto_orient'])
 
         num_images = prefs.get('numImages', 0)
         self.w.num_images.set_text(str(num_images))
