@@ -27,6 +27,24 @@ This plugin is not usually configured to be closeable, but the user can
 make it so by setting the "closeable" setting to True in the configuration
 file--then Close and Help buttons will be added to the bottom of the UI.
 
+**Excluding images from Thumbs**
+
+.. note:: This also controls the behavior of ``Contents``.
+
+Although the default behavior is for every image that is loaded into the
+reference viewer to show up in ``Thumbs``, there may be cases where this
+is undesirable (e.g., when there are many images being loaded at a
+periodic rate by some automated process).  In such cases there are two
+mechanisms for suppressing certain images from showing up in ``Thumbs``:
+
+* Assigning the "genthumb" setting to False in a channel's settings
+  (for example from the ``Preferences`` plugin, under the "General"
+  settings) will exclude the channel itself and any of its images.
+* Setting the "nothumb" keyword in the metadata of an image wrapper
+  (not the FITS header, but by e.g., ``image.set(nothumb=True)``)
+  will exclude that particular image from ``Thumbs``, even if the
+  "genthumb" setting is True for that channel.
+
 """
 import os
 import math
@@ -82,6 +100,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
                                    label_cutoff='right',
                                    highlight_tracks_keyboard_focus=True,
                                    label_font_color='white',
+                                   label_font='sans condensed',
                                    label_font_size=10,
                                    label_bg_color='lightgreen',
                                    autoload_visible_thumbs=False,
@@ -188,6 +207,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         bd.enable_pan(True)
         bd.enable_zoom(False)
         bd.enable_cmap(False)
+        bd.get_settings().set(scroll_pan_lock_x=True)
 
         # remap some bindings for pan mode into no mode needed
         bm = c_v.get_bindmap()
@@ -647,9 +667,12 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             # no image has the focus
             new_highlight = set([])
 
+        nothumb = (image.get('nothumb', False) or
+                   not channel.settings.get('genthumb', True))
+
         with self.thumblock:
             #if not self.have_thumbnail(channel.fitsimage, image):
-            if thumbkey not in self.thumb_dict:
+            if thumbkey not in self.thumb_dict and not nothumb:
                 # No memory of this thumbnail, so regenerate it
                 if not self._add_image(self.fv, chname, image, None):
                     return
@@ -777,7 +800,12 @@ class Thumbs(GingaPlugin.GlobalPlugin):
 
             # Save a thumbnail for future browsing
             if save_thumb and info.path is not None:
-                self._save_thumb(thumb_image, bnch)
+                try:
+                    self._save_thumb(thumb_image, bnch)
+                except Exception as e:
+                    # if we can't persist the thumbnail, don't let that
+                    # stop everything
+                    self.logger.error("Couldn't persist thumbnail: {}".format(e))
 
             self.update_thumbnail(thumbkey, thumb_image, metadata)
 
@@ -857,7 +885,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         a, b = 2, 0
         for line in lines:
             text = canvas.dc.Text(a, b, text=line, color='black',
-                                  fontsize=fontsize)
+                                  font='sans condensed', fontsize=fontsize)
             text.crdmap = crdmap
             l.append(text)
             txt_wd, txt_ht = thumbs_viewer.renderer.get_dimensions(text)
@@ -877,6 +905,16 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             rect.x1 -= off
             rect.x2 -= off
             point.x -= off
+        if rect.y1 < y3:
+            off = y3 - rect.y1
+            rect.y1 += off
+            rect.y2 += off
+            point.y += off
+        if rect.y2 > y4:
+            off = rect.y2 - y4
+            rect.y1 -= off
+            rect.y2 -= off
+            point.y -= off
 
         return obj
 
@@ -1059,8 +1097,9 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         # calc in data coords
         thumb_height = self.thumb_width
         twd_vplus = thumb_height + text_ht + self.thumb_vsep
-        row1 = int(math.floor(abs(y1) / twd_vplus) - 2)
-        row2 = int(math.ceil(abs(y2) / twd_vplus) + 2)
+        # remember, Y-axis is flipped
+        row1 = max(0, int(math.floor(y1 / twd_vplus) - 2))
+        row2 = max(row1 + 1, int(math.ceil(y2 / twd_vplus) + 2))
         self.logger.debug("row1, row2 = %d, %d" % (row1, row2))
         return row1, row2
 
@@ -1245,6 +1284,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         canvas = self.canvas
         fg = self.settings.get('label_font_color', 'white')
         bg = self.settings.get('label_bg_color', 'lightgreen')
+        fontname = self.settings.get('label_font', 'sans condensed')
         fontsize = self.settings.get('label_font_size', 10)
 
         # Shorten thumbnail label, if requested
@@ -1264,7 +1304,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         l2 = []
         color = bg if thumbkey in self.re_hilite_set else fg
         namelbl = canvas.dc.Text(xt, yt, text=thumbname, color=color,
-                                 fontsize=fontsize, coord='data')
+                                 font=fontname, fontsize=fontsize,
+                                 coord='data')
         l2.append(namelbl)
 
         image = canvas.dc.Image(xi, yi, thumb_image, alpha=1.0,
