@@ -5,7 +5,6 @@
 # Please see the file LICENSE.txt for details.
 #
 import numpy as np
-from astropy.table import Table
 
 from ginga.BaseImage import ViewerObjectBase, Header
 from ginga.util import wcsmod
@@ -45,7 +44,10 @@ class AstroTable(ViewerObjectBase):
 
         self._data = data_ap
         self.naxispath = []
+        self.colnames = []
 
+        # TODO: How to handle table with WCS data? For example, spectrum
+        #       table may store dispersion solution as WCS.
         # wcsclass specifies a pluggable WCS module
         if wcsclass is None:
             wcsclass = wcsmod.WCS
@@ -57,12 +59,6 @@ class AstroTable(ViewerObjectBase):
             ioclass = io_fits.fitsLoaderClass
         self.io = ioclass(self.logger)
 
-        # TODO: How to handle table with WCS data? For example, spectrum
-        #       table may store dispersion solution as WCS.
-        #if metadata is not None:
-        #    header = self.get_header()
-        #    self.wcs.load_header(header)
-
     @property
     def rows(self):
         tab_a = self._get_data()
@@ -70,8 +66,7 @@ class AstroTable(ViewerObjectBase):
 
     @property
     def columns(self):
-        tab_a = self._get_data()
-        return len(tab_a.colnames)
+        return len(self.colnames)
 
     def get_size(self):
         return (self.columns, self.rows)
@@ -82,20 +77,40 @@ class AstroTable(ViewerObjectBase):
     def _get_data(self):
         return self._data
 
-    def get_header(self, create=True, **kwargs):
-        try:
-            # By convention, the fits header is stored in a dictionary
-            # under the metadata keyword 'header'
-            displayhdr = self.metadata['header']
-
-        except KeyError as e:
+    def get_header(self, create=True, include_primary_header=None):
+        # By convention, the fits header is stored in a dictionary
+        # under the metadata keyword 'header'
+        if 'header' not in self:
             if not create:
-                raise e
+                # TODO: change to ValueError("No header found")
+                raise KeyError('header')
+
+            hdr = AstroTableHeader()
+            self.set(header=hdr)
+        else:
+            hdr = self['header']
+
+        if include_primary_header is None:
+            include_primary_header = self.get('inherit_primary_header', False)
+
+        # If it was saved during loading, by convention, the primary fits
+        # header is stored in a dictionary under the keyword 'primary_header'
+        if include_primary_header and self.has_primary_header():
+            # Inherit PRIMARY header for display but keep metadata intact
             displayhdr = AstroTableHeader()
-            self.metadata['header'] = displayhdr
+            displayhdr.merge(hdr)
+            primary_hdr = self['primary_header']
+            displayhdr.merge(primary_hdr, override_keywords=False)
+        else:
+            # Normal, separate header
+            displayhdr = hdr
 
         return displayhdr
 
+    def has_primary_header(self):
+        primary_hdr = self.get('primary_header', None)
+        return isinstance(primary_hdr, AstroTableHeader)
+
     def set_data(self, data_ap, metadata=None):
         """Use this method to SHARE (not copy) the incoming table.
         """
@@ -118,24 +133,10 @@ class AstroTable(ViewerObjectBase):
         return (0, 0)
 
     def load_hdu(self, hdu, fobj=None, **kwargs):
-        self.clear_metadata()
+        if self.io is None:
+            raise TableError("No IO loader defined")
 
-        # These keywords might be provided but not used.
-        if 'inherit_primary_header' in kwargs:
-            kwargs.pop('inherit_primary_header')
-
-        ahdr = self.get_header()
-
-        if 'format' not in kwargs:
-            kwargs['format'] = 'fits'
-
-        try:
-            self._data = Table.read(hdu, **kwargs)
-            ahdr.update(hdu.header)
-
-        except Exception as e:
-            self.logger.error("Error reading table from hdu: {0}".format(
-                str(e)))
+        self.io.load_hdu(hdu, fobj=fobj, dstobj=self, **kwargs)
 
         # TODO: Try to make a wcs object on the header
         #self.wcs.load_header(hdu.header, fobj=fobj)
@@ -149,5 +150,3 @@ class AstroTable(ViewerObjectBase):
     def get_thumbnail(self, length):
         thumb_np = np.eye(length)
         return thumb_np
-
-#END
