@@ -27,18 +27,16 @@ def use(name):
     elif name.startswith('qt') or name.startswith('pyside'):
         family = 'qt'
         if name == 'qt':
-            name = 'qt4'
-        assert name in ('qt4', 'pyside', 'qt5'), \
-            ToolKitError("ToolKit '%s' not supported!" % (name))
+            name = 'qt5'
+        if name not in ('qt4', 'pyside', 'qt5', 'pyside2'):
+            raise ToolKitError("ToolKit '%s' not supported!" % (name))
 
     elif name.startswith('gtk'):
         # default for "gtk" is gtk3
         if name in ('gtk', 'gtk3'):
             name = 'gtk3'
             family = 'gtk3'
-        elif name in ('gtk2',):
-            family = 'gtk'
-        assert name in ('gtk2', 'gtk3'), \
+        assert name in ['gtk3'], \
             ToolKitError("ToolKit '%s' not supported!" % (name))
 
     elif name.startswith('tk'):
@@ -67,7 +65,7 @@ def get_family():
 
 def get_rv_toolkits():
     """Returns a list of reference viewer supported toolkits."""
-    return ['qt4', 'qt5', 'pyside', 'gtk2', 'gtk3', 'pg']
+    return ['qt4', 'qt5', 'pyside', 'pyside2', 'gtk3', 'pg']
 
 
 def choose():
@@ -77,9 +75,6 @@ def choose():
         try:
             from ginga.gtkw3 import GtkHelp  # noqa
         except ImportError:
-            try:
-                from ginga.gtkw import GtkHelp  # noqa
-            except ImportError:
-                raise ImportError("qt or gtk variants not found")
+            raise ImportError("qt or gtk variants not found")
 
 # END
