@@ -13,7 +13,6 @@ _use = None
 
 
 def use(pkgname):
-    global have_opencl, trcalc_cl
     global _use
 
     if pkgname == 'opencv':
@@ -22,16 +21,6 @@ def use(pkgname):
     elif pkgname == 'pillow':
         _use = 'pillow'
 
-    elif pkgname == 'opencl':
-        try:
-            from ginga.opencl import CL
-            have_opencl = True
-            _use = 'opencl'
-
-            trcalc_cl = CL.CL('trcalc.cl')
-        except Exception as e:
-            raise ImportError(e)
-
 
 have_opencv = False
 try:
@@ -55,20 +44,6 @@ try:
 except ImportError:
     pass
 
-have_opencl = False
-trcalc_cl = None
-try:
-    # optional opencl package speeds up certain operations, especially
-    # rotation
-    # TEMP: pyopencl prompts users if it can't determine which device
-    #       to use for acceleration.
-    # --> temporarily disable, can enable using use() function above
-    #use('opencl')
-    pass
-
-except ImportError:
-    pass
-
 have_pillow = False
 try:
     # do we have Python Imaging Library available?
@@ -90,20 +65,8 @@ try:
 except ImportError:
     pass
 
-have_numexpr = False
-try:
-    # optional numexpr package speeds up certain combined numpy array
-    # operations, useful to have
-    import numexpr as ne
-    have_numexpr = True
-
-except ImportError:
-    pass
-
 # For testing
-#have_numexpr = False
 #have_opencv = False
-#have_opencl = False
 #have_pillow = False
 
 
@@ -176,7 +139,7 @@ def rotate_clip(data_np, theta_deg, rotctr_x=None, rotctr_y=None,
         if logger is not None:
             logger.debug("rotating with OpenCv")
         # opencv is fastest
-        M = cv2.getRotationMatrix2D((rotctr_y, rotctr_x), theta_deg, 1)
+        M = cv2.getRotationMatrix2D((rotctr_x, rotctr_y), theta_deg, 1)
 
         newdata = cv2.warpAffine(data_np, M, (wd, ht))
         new_ht, new_wd = newdata.shape[:2]
@@ -202,20 +165,6 @@ def rotate_clip(data_np, theta_deg, rotctr_x=None, rotctr_y=None,
             Exception("rotated cutout is %dx%d original=%dx%d" % (
                 new_wd, new_ht, wd, ht))
 
-    elif dtype == np.uint8 and have_opencl and _use in (None, 'opencl'):
-        if logger is not None:
-            logger.debug("rotating with OpenCL")
-        # opencl is very close, sometimes better, sometimes worse
-        if (data_np.dtype == np.uint8) and (len(data_np.shape) == 3):
-            # special case for 3D RGB images
-            newdata = trcalc_cl.rotate_clip_uint32(data_np, theta_deg,
-                                                   rotctr_x, rotctr_y,
-                                                   out=out)
-        else:
-            newdata = trcalc_cl.rotate_clip(data_np, theta_deg,
-                                            rotctr_x, rotctr_y,
-                                            out=out)
-
     else:
         if logger is not None:
             logger.debug("rotating with numpy")
@@ -225,21 +174,17 @@ def rotate_clip(data_np, theta_deg, rotctr_x=None, rotctr_y=None,
         cos_t = np.cos(np.radians(theta_deg))
         sin_t = np.sin(np.radians(theta_deg))
 
-        if have_numexpr:
-            ap = ne.evaluate("(xi * cos_t) - (yi * sin_t) + rotctr_x")
-            bp = ne.evaluate("(xi * sin_t) + (yi * cos_t) + rotctr_y")
-        else:
-            ap = (xi * cos_t) - (yi * sin_t) + rotctr_x
-            bp = (xi * sin_t) + (yi * cos_t) + rotctr_y
+        ap = (xi * cos_t) - (yi * sin_t) + rotctr_x
+        bp = (xi * sin_t) + (yi * cos_t) + rotctr_y
 
-        #ap = np.rint(ap).clip(0, wd-1).astype(np.int)
-        #bp = np.rint(bp).clip(0, ht-1).astype(np.int)
+        #ap = np.rint(ap).clip(0, wd-1).astype(int)
+        #bp = np.rint(bp).clip(0, ht-1).astype(int)
         # Optomizations to reuse existing intermediate arrays
         np.rint(ap, out=ap)
-        ap = ap.astype(np.int, copy=False)
+        ap = ap.astype(int, copy=False)
         ap.clip(0, wd - 1, out=ap)
         np.rint(bp, out=bp)
-        bp = bp.astype(np.int, copy=False)
+        bp = bp.astype(int, copy=False)
         bp.clip(0, ht - 1, out=bp)
 
         if out is not None:
@@ -274,31 +219,19 @@ def rotate(data_np, theta_deg, rotctr_x=None, rotctr_y=None, pad=20,
     # Find center of new data array
     ncx, ncy = new_wd // 2, new_ht // 2
 
-    if have_opencl and _use == 'opencl':
-        if logger is not None:
-            logger.debug("rotating with OpenCL")
-        # find offsets of old image in new image
-        dx, dy = ncx - ocx, ncy - ocy
+    # Overlay the old image on the new (blank) image
+    ldx, rdx = min(ocx, ncx), min(wd - ocx, ncx)
+    bdy, tdy = min(ocy, ncy), min(ht - ocy, ncy)
 
-        newdata = trcalc_cl.rotate(data_np, theta_deg,
-                                   rotctr_x=rotctr_x, rotctr_y=rotctr_y,
-                                   clip_val=0, out=None,
-                                   out_wd=new_wd, out_ht=new_ht,
-                                   out_dx=dx, out_dy=dy)
-    else:
-        # Overlay the old image on the new (blank) image
-        ldx, rdx = min(ocx, ncx), min(wd - ocx, ncx)
-        bdy, tdy = min(ocy, ncy), min(ht - ocy, ncy)
+    # TODO: fill with a different value?
+    newdata = np.zeros(dims, dtype=data_np.dtype)
+    newdata[ncy - bdy:ncy + tdy, ncx - ldx:ncx + rdx] = \
+        data_np[ocy - bdy:ocy + tdy, ocx - ldx:ocx + rdx]
 
-        # TODO: fill with a different value?
-        newdata = np.zeros(dims, dtype=data_np.dtype)
-        newdata[ncy - bdy:ncy + tdy, ncx - ldx:ncx + rdx] = \
-            data_np[ocy - bdy:ocy + tdy, ocx - ldx:ocx + rdx]
-
-        # Now rotate with clip as usual
-        newdata = rotate_clip(newdata, theta_deg,
-                              rotctr_x=rotctr_x, rotctr_y=rotctr_y,
-                              out=newdata)
+    # Now rotate with clip as usual
+    newdata = rotate_clip(newdata, theta_deg,
+                          rotctr_x=rotctr_x, rotctr_y=rotctr_y,
+                          out=newdata)
     return newdata
 
 
@@ -323,9 +256,9 @@ def get_scaled_cutout_wdht_view(shp, x1, y1, x2, y2, new_wd, new_ht):
     # Make indexes and scale them
     # Is there a more efficient way to do this?
     xi = np.clip(x1 + np.arange(0, new_wd) * iscale_x,
-                 0, max_x).astype(np.int, copy=False)
+                 0, max_x).astype(int, copy=False)
     yi = np.clip(y1 + np.arange(0, new_ht) * iscale_y,
-                 0, max_y).astype(np.int, copy=False)
+                 0, max_y).astype(int, copy=False)
     wd, ht = xi.size, yi.size
 
     # bounds check against shape (to protect future data access)
@@ -384,11 +317,11 @@ def get_scaled_cutout_wdhtdp_view(shp, p1, p2, new_dims):
         iscale_z = float(old_dp) / float(new_dp)
 
     xi = np.clip(x1 + np.arange(0, new_wd) * iscale_x,
-                 0, max_x).astype(np.int, copy=False)
+                 0, max_x).astype(int, copy=False)
     yi = np.clip(y1 + np.arange(0, new_ht) * iscale_y,
-                 0, max_y).astype(np.int, copy=False)
+                 0, max_y).astype(int, copy=False)
     zi = np.clip(z1 + np.arange(0, new_dp) * iscale_z,
-                 0, max_z).astype(np.int, copy=False)
+                 0, max_z).astype(int, copy=False)
     wd, ht, dp = xi.size, yi.size, zi.size
 
     # bounds check against shape (to protect future data access)
@@ -472,20 +405,6 @@ def get_scaled_cutout_wdht(data_np, x1, y1, x2, y2, new_wd, new_ht,
         ht, wd = newdata.shape[:2]
         scale_x, scale_y = float(wd) / old_wd, float(ht) / old_ht
 
-    elif (data_np.dtype == np.uint8 and have_opencl and
-            interpolation in ('basic', 'nearest') and
-            open_cl_ok and _use in (None, 'opencl')):
-        # opencl is almost as fast or sometimes faster, but currently
-        # we only support nearest neighbor
-        if logger is not None:
-            logger.debug("resizing with OpenCL")
-        old_wd, old_ht = max(x2 - x1 + 1, 1), max(y2 - y1 + 1, 1)
-        scale_x, scale_y = float(new_wd) / old_wd, float(new_ht) / old_ht
-
-        newdata, (scale_x, scale_y) = trcalc_cl.get_scaled_cutout_basic(data_np,
-                                                                        x1, y1, x2, y2,
-                                                                        scale_x, scale_y)
-
     elif interpolation not in ('basic', 'nearest'):
         raise ValueError("Interpolation method not supported: '%s'" % (
             interpolation))
@@ -583,13 +502,6 @@ def get_scaled_cutout_basic(data_np, x1, y1, x2, y2, scale_x, scale_y,
         ht, wd = newdata.shape[:2]
         scale_x, scale_y = float(wd) / old_wd, float(ht) / old_ht
 
-    elif (have_opencl and interpolation in ('basic', 'nearest') and
-            open_cl_ok and _use in (None, 'opencl')):
-        if logger is not None:
-            logger.debug("resizing with OpenCL")
-        newdata, (scale_x, scale_y) = trcalc_cl.get_scaled_cutout_basic(
-            data_np, x1, y1, x2, y2, scale_x, scale_y)
-
     elif interpolation not in ('basic', 'nearest'):
         raise ValueError("Interpolation method not supported: '%s'" % (
             interpolation))
@@ -827,10 +739,7 @@ def overlay_image_2d_np(dstarr, pos, srcarr, dst_order='RGBA',
     else:
         # calculate alpha blending
         #   Co = CaAa + CbAb(1 - Aa)
-        if have_numexpr:
-            _dst[:, :, :] = ne.evaluate("(alpha * _src) + (1.0 - alpha) * _dst")
-        else:
-            _dst[:, :, :] = (alpha * _src) + (1.0 - alpha) * _dst
+        _dst[:, :, :] = (alpha * _src) + (1.0 - alpha) * _dst
 
     return dstarr
 
@@ -958,10 +867,7 @@ def overlay_image_3d(dstarr, pos, srcarr, dst_order='RGBA', src_order='RGBA',
     else:
         # calculate alpha blending
         #   Co = CaAa + CbAb(1 - Aa)
-        if have_numexpr:
-            _dst[:, :, :, :] = ne.evaluate("(alpha * _src) + (1.0 - alpha) * _dst")
-        else:
-            _dst[:, :, :, :] = (alpha * _src) + (1.0 - alpha) * _dst
+        _dst[:, :, :, :] = (alpha * _src) + (1.0 - alpha) * _dst
 
     return dstarr
 
@@ -980,27 +886,33 @@ def reorder_image(dst_order, src_arr, src_order):
     """
     depth = src_arr.shape[2]
     if depth != len(src_order):
-        raise ValueError("src_order (%s) does not match array depth (%d)" % (
-            src_order, depth))
+        if len(dst_order.replace('A', '')) != len(src_order.replace('A', '')):
+            raise ValueError("src_order (%s) does not match array depth (%d)" % (
+                src_order, depth))
 
     bands = []
     if dst_order == src_order:
         return np.ascontiguousarray(src_arr)
 
-    elif 'A' not in dst_order or 'A' in src_order:
+    missing = set(dst_order) - set(src_order)
+    if len(missing) == 0:
         # <-- we don't have to add an alpha plane, just create a new view
         idx = np.array([src_order.index(c) for c in dst_order])
         return np.ascontiguousarray(src_arr[..., idx])
 
-    else:
-        # <-- dst order requires missing alpha channel
-        indexes = [src_order.index(c) for c in dst_order.replace('A', '')]
-        bands = [src_arr[..., idx, np.newaxis] for idx in indexes]
-        ht, wd = src_arr.shape[:2]
-        dst_type = src_arr.dtype
-        dst_max_val = np.iinfo(dst_type).max
-        alpha = np.full((ht, wd, 1), dst_max_val, dtype=dst_type)
-        bands.insert(dst_order.index('A'), alpha)
+    if missing != set(['A']):
+        missing = list(missing - set(['A']))
+        raise ValueError("source array missing channels ({}) needed in "
+                         "destination array ({})".format(src_order, dst_order))
+
+    # <-- dst order requires missing alpha channel
+    indexes = [src_order.index(c) for c in dst_order.replace('A', '')]
+    bands = [src_arr[..., idx, np.newaxis] for idx in indexes]
+    ht, wd = src_arr.shape[:2]
+    dst_type = src_arr.dtype
+    dst_max_val = np.iinfo(dst_type).max
+    alpha = np.full((ht, wd, 1), dst_max_val, dtype=dst_type)
+    bands.insert(dst_order.index('A'), alpha)
 
     return np.concatenate(bands, axis=-1)
 
@@ -1111,3 +1023,100 @@ def check_native_byteorder(data_np):
 
     return ((dt.startswith('>') and sys.byteorder == 'little') or
             (dt.startswith('<') and sys.byteorder == 'big'))
+
+
+def cutout_data(data, x1, y1, x2, y2, xstep=1, ystep=1, z=None,
+                astype=None):
+    """Cut out data area based on bounded coordinates.
+
+    Parameters
+    ----------
+    x1, y1 : int
+        Coordinates defining the minimum corner to be cut out
+
+    x2, y2 : int
+        Coordinates *one greater* than the maximum corner
+
+    xstep, ystep : int
+        Step values for skip intervals in the cutout region
+
+    z : int
+        Value for a depth (slice) component for color images
+
+    astype :
+
+    Note that the coordinates for `x2`, `y2` are *outside* the
+    cutout region, similar to slicing parameters in Python.
+    """
+    view = np.s_[y1:y2:ystep, x1:x2:xstep]
+    data_np = data[view]
+    if z is not None and len(data_np.shape) > 2:
+        data_np = data_np[..., z]
+    if astype:
+        data_np = data_np.astype(astype, copy=False)
+    return data_np
+
+
+def cutout_adjust(data, x1, y1, x2, y2, xstep=1, ystep=1, z=0, astype=None):
+    """Like `cutout_data`, but adjusts coordinates `x1`, `y1`, `x2`, `y2`
+    to be inside the data area if they are not already.  It tries to
+    preserve the width and height of the region, so e.g. (-2, -2, 5, 5)
+    could become (0, 0, 7, 7)
+    """
+    height, width = data.shape[:2]
+    dx = x2 - x1
+    dy = y2 - y1
+
+    if x1 < 0:
+        x1, x2 = 0, dx
+    else:
+        if x2 >= width:
+            x2 = width
+            x1 = x2 - dx
+
+    if y1 < 0:
+        y1, y2 = 0, dy
+    else:
+        if y2 >= height:
+            y2 = height
+            y1 = y2 - dy
+
+    data = cutout_data(data, x1, y1, x2, y2, xstep=xstep, ystep=ystep,
+                       z=z, astype=astype)
+    return (data, x1, y1, x2, y2)
+
+
+def cutout_radius(data, x, y, radius, xstep=1, ystep=1, astype=None):
+    return cutout_adjust(data, x - radius, y - radius,
+                         x + radius + 1, y + radius + 1,
+                         xstep=xstep, ystep=ystep, astype=astype)
+
+
+def guess_order(shape):
+    if len(shape) <= 2:
+        order = 'M'
+    else:
+        depth = shape[-1]
+        if depth == 1:
+            order = 'M'
+        elif depth == 2:
+            order = 'MA'
+        elif depth == 3:
+            order = 'RGB'
+        elif depth == 4:
+            order = 'RGBA'
+
+    return order
+
+
+def get_aspect(shape):
+    return shape[1] / shape[0]
+
+
+def calc_aspect_str(wd, ht):
+    # calculate the aspect ratio given by width and height and make
+    # string of the form "x:y"
+    gcd = np.gcd(wd, ht)
+    _wd, _ht = int(wd / gcd), int(ht / gcd)
+    _as = str(_wd) + ':' + str(_ht)
+    return _as
