@@ -18,6 +18,7 @@ any images.  Otherwise Ginga will try to pick one for you.
 """
 import re
 import numpy as np
+import warnings
 
 from ginga.AstroImage import AstroImage, AstroHeader
 from ginga.table.AstroTable import AstroTable
@@ -31,6 +32,7 @@ fitsLoaderClass = None
 
 try:
     from astropy.io import fits as pyfits
+    from astropy.table import Table
     have_astropy = True
 except ImportError:
     have_astropy = False
@@ -56,8 +58,8 @@ def use(fitspkg, raise_err=True):
             return True
 
         elif raise_err:
-            raise("Error importing 'astropy.io.fits'; "
-                  "please check installation")
+            raise ImportError("Error importing 'astropy.io.fits'; "
+                              "please check installation")
 
         return False
 
@@ -68,8 +70,8 @@ def use(fitspkg, raise_err=True):
             return True
 
         elif raise_err:
-            raise("Error importing 'fitsio'; "
-                  "please check installation")
+            raise ImportError("Error importing 'fitsio'; "
+                              "please check installation")
 
         return False
 
@@ -219,7 +221,10 @@ class PyFitsFileHandler(BaseFitsFileHandler):
         super(PyFitsFileHandler, self).__init__(logger)
         self.kind = 'pyfits'
 
-    def fromHDU(self, hdu, ahdr):
+    def copy_header(self, hdu, ahdr):
+        """Copy a FITS header from an astropy.io.fits.PrimaryHDU object
+        into a ginga.AstroImage.AstroHeader object.
+        """
         header = hdu.header
         if hasattr(header, 'cards'):
             # newer astropy.io.fits don't have ascardlist()
@@ -246,18 +251,50 @@ class PyFitsFileHandler(BaseFitsFileHandler):
 
         return None
 
-    def load_hdu(self, hdu, dstobj=None, **kwargs):
+    def load_hdu(self, hdu, dstobj=None, fobj=None, naxispath=None,
+                 save_primary_header=False, inherit_primary_header=False,
+                 **kwargs):
+        if fobj is None:
+            fobj = self.fits_f
+
+        save_primary_header = save_primary_header or inherit_primary_header
+        # Set PRIMARY header
+        if save_primary_header and fobj is not None:
+            primary_hdr = AstroHeader()
+            self.copy_header(fobj[0], primary_hdr)
+        else:
+            primary_hdr = None
 
         typ = self.get_hdu_type(hdu)
         if typ == 'image':
 
             if dstobj is None:
                 dstobj = AstroImage(logger=self.logger)
-                dstobj.load_hdu(hdu, **kwargs)
-
             else:
-                # TODO: migrate code from AstroImage to here
-                dstobj.load_hdu(hdu, **kwargs)
+                if not isinstance(dstobj, AstroImage):
+                    raise ValueError("dstobj needs to be a type of AstroImage")
+                dstobj.clear_metadata()
+
+            # this seems to be necessary now for some fits files...
+            try:
+                hdu.verify('fix')
+
+            except Exception as e:
+                # Let's hope for the best!
+                self.logger.warning("Problem verifying fits HDU: {}".format(e))
+
+            # collect HDU header
+            ahdr = dstobj.get_header()
+            self.copy_header(hdu, ahdr)
+            dstobj.set(primary_header=primary_hdr,
+                       inherit_primary_header=inherit_primary_header)
+
+            dstobj.setup_data(hdu.data, naxispath=naxispath)
+
+            # Try to make a wcs object on the header
+            wcs = getattr(dstobj, 'wcs', None)
+            if wcs is not None:
+                wcs.load_header(hdu.header, fobj=fobj)
 
         elif typ == 'table':
             # <-- data may be a table
@@ -274,11 +311,31 @@ class PyFitsFileHandler(BaseFitsFileHandler):
 
             if dstobj is None:
                 dstobj = AstroTable(logger=self.logger)
+            else:
+                if not isinstance(dstobj, AstroTable):
+                    raise ValueError("dstobj needs to be a type of AstroTable")
+                dstobj.clear_metadata()
+
+            dstobj.kind = 'table-astropy'
 
             self.logger.debug('Attempting to load table from FITS')
 
-            # TODO: migrate code from AstroTable to here
-            dstobj.load_hdu(hdu, **kwargs)
+            ahdr = dstobj.get_header()
+            self.copy_header(hdu, ahdr)
+            dstobj.set(primary_header=primary_hdr,
+                       inherit_primary_header=inherit_primary_header)
+
+            if 'format' not in kwargs:
+                kwargs['format'] = 'fits'
+
+            try:
+                data = Table.read(hdu, **kwargs)
+                dstobj.set_data(data)
+                dstobj.colnames = list(data.colnames)
+
+            except Exception as e:
+                self.logger.error("Error reading table from hdu: {0}".format(e),
+                                  exc_info=True)
 
         else:
             raise FITSError("I don't know how to read this HDU")
@@ -288,13 +345,15 @@ class PyFitsFileHandler(BaseFitsFileHandler):
         return dstobj
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
+                  save_primary_header=False, inherit_primary_header=False,
                   **kwargs):
-        inherit_primary_header = kwargs.pop('inherit_primary_header', False)
+
         opener = self.get_factory()
         opener.open_file(filespec, memmap=memmap, **kwargs)
         try:
             return opener.get_hdu(
                 numhdu, dstobj=dstobj,
+                save_primary_header=save_primary_header,
                 inherit_primary_header=inherit_primary_header)
         finally:
             opener.close()
@@ -472,6 +531,12 @@ class PyFitsFileHandler(BaseFitsFileHandler):
     def save_as_file(self, filepath, data, header, **kwargs):
         self.write_fits(filepath, data, header, **kwargs)
 
+    def fromHDU(self, hdu, ahdr):
+        warnings.warn("fromHDU will be deprecated in the next release--"
+                      "use copy_header instead",
+                      DeprecationWarning)
+        return self.copy_header(hdu, ahdr)
+
 
 class FitsioFileHandler(BaseFitsFileHandler):
 
@@ -491,7 +556,7 @@ class FitsioFileHandler(BaseFitsFileHandler):
         self.hdu_info = []
         self.hdu_db = {}
 
-    def fromHDU(self, hdu, ahdr):
+    def copy_header(self, hdu, ahdr):
         header = hdu.read_header()
         for d in header.records():
             if len(d['name']) == 0:
@@ -509,42 +574,80 @@ class FitsioFileHandler(BaseFitsFileHandler):
 
         return None
 
-    def load_hdu(self, hdu, dstobj=None, **kwargs):
+    def load_hdu(self, hdu, dstobj=None, fobj=None, naxispath=None,
+                 save_primary_header=False, inherit_primary_header=False,
+                 **kwargs):
+        if fobj is None:
+            fobj = self.fits_f
+
+        save_primary_header = save_primary_header or inherit_primary_header
+        # Set PRIMARY header
+        if save_primary_header and fobj is not None:
+            primary_hdr = AstroHeader()
+            self.copy_header(fobj[0], primary_hdr)
+        else:
+            primary_hdr = None
+
         typ = self.get_hdu_type(hdu)
 
         if typ == 'image':
             # <-- data is an image
-            ahdr = AstroHeader()
-            self.fromHDU(hdu, ahdr)
+            if dstobj is None:
+                dstobj = AstroImage(logger=self.logger)
+            else:
+                if not isinstance(dstobj, AstroImage):
+                    raise ValueError("dstobj needs to be a type of AstroImage")
+                dstobj.clear_metadata()
 
-            metadata = dict(header=ahdr)
+            ahdr = dstobj.get_header()
+            self.copy_header(hdu, ahdr)
+            dstobj.set(primary_header=primary_hdr,
+                       inherit_primary_header=inherit_primary_header)
 
             data = hdu.read()
 
-            if dstobj is None:
-                dstobj = AstroImage(logger=self.logger)
-                dstobj.load_data(data, metadata=metadata)
+            dstobj.setup_data(data, naxispath=naxispath)
 
-            else:
-                dstobj.load_data(data, metadata=metadata)
+            # Try to make a wcs object on the header
+            wcs = getattr(dstobj, 'wcs', None)
+            if wcs is not None:
+                # NOTE: WCS packages don't know what to do with fitsio objects
+                wcs.load_header(ahdr.asdict(), fobj=None)
 
         elif typ == 'table':
             # <-- data is a table
-            raise FITSError(
-                "FITS tables are not yet readable using ginga/fitsio")
+            if dstobj is None:
+                dstobj = AstroTable(logger=self.logger)
+            else:
+                if not isinstance(dstobj, AstroTable):
+                    raise ValueError("dstobj needs to be a type of AstroTable")
+                dstobj.clear_metadata()
+
+            ahdr = dstobj.get_header()
+            self.copy_header(hdu, ahdr)
+            dstobj.set(primary_header=primary_hdr,
+                       inherit_primary_header=inherit_primary_header)
+
+            dstobj.kind = 'table-fitsio'
+
+            data = hdu.read()
+            dstobj.colnames = list(data.dtype.fields)
+            dstobj.set_data(data)
 
         dstobj.io = self
 
         return dstobj
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
+                  save_primary_header=False, inherit_primary_header=False,
                   **kwargs):
-        inherit_primary_header = kwargs.pop('inherit_primary_header', False)
+
         opener = self.get_factory()
         opener.open_file(filespec, memmap=memmap, **kwargs)
         try:
             return opener.get_hdu(
                 numhdu, dstobj=dstobj,
+                save_primary_header=save_primary_header,
                 inherit_primary_header=inherit_primary_header)
         finally:
             opener.close()
@@ -687,27 +790,29 @@ class FitsioFileHandler(BaseFitsFileHandler):
         return dstobj
 
     def create_fits(self, data, header):
-        fits_f = pyfits.HDUList()
-        hdu = pyfits.PrimaryHDU()
-        hdu.data = data
-
-        for kwd in header.keys():
-            card = header.get_card(kwd)
-            hdu.header.update(card.key, card.value, comment=card.comment)
-
-        fits_f.append(hdu)
-        return fits_f
+        raise NotImplementedError("This function is not implemented for the 'fitsio' wrapper")
 
     def write_fits(self, path, data, header):
         fits_f = fitsio.FITS(path, 'rw')
 
-        fits_f = self.create_fits(data, header)
-        fits_f.writeto(path, output_verify='fix')
+        hlist = []
+        for key in header.keys():
+            card = header.get_card(key)
+            hlist.append(dict(name=key, value=card.value,
+                              comment=card.comment))
+
+        fits_f.write(data, header=hlist)
         fits_f.close()
 
     def save_as_file(self, filepath, data, header, **kwargs):
         self.write_fits(filepath, data, header, **kwargs)
 
+    def fromHDU(self, hdu, ahdr):
+        warnings.warn("fromHDU will be deprecated in the next release--"
+                      "use copy_header instead",
+                      DeprecationWarning)
+        return self.copy_header(hdu, ahdr)
+
 
 if not fits_configured:
     if have_astropy:
@@ -735,6 +840,3 @@ def load_file(filepath, idx=None, logger=None, **kwargs):
     """
     opener = get_fitsloader(logger=logger)
     return opener.load_file(filepath, numhdu=idx, **kwargs)
-
-
-# END
