@@ -47,6 +47,7 @@ class ViewerImageProxy:
     * get_size
     * get_depth
     * get_center
+    * get_minmax
     * get_data_xy
     * info_xy
     * pixtoradec
@@ -182,7 +183,7 @@ class ViewerImageProxy:
     # ----- for compatibility with BaseImage objects -----
 
     def cutout_data(self, x1, y1, x2, y2, xstep=1, ystep=1, z=0,
-                    astype=np.float, fill_value=np.NaN):
+                    astype=float, fill_value=np.NaN):
         """Cut out data area based on rectangular coordinates.
 
         Parameters
@@ -220,7 +221,7 @@ class ViewerImageProxy:
         t1 = time.time()
 
         if astype is None:
-            astype = np.float
+            astype = float
         if fill_value is None:
             fill_value = np.NaN
 
@@ -237,7 +238,7 @@ class ViewerImageProxy:
         data_np = np.full((y_len, x_len), fill_value, dtype=astype)
 
         # calculate pixel containment indexes in cutout bbox
-        yi, xi = np.mgrid[y1:y2, x1:x2]
+        yi, xi = np.mgrid[y1:y2:ystep, x1:x2:xstep]
         pts = np.asarray((xi, yi)).T
 
         canvas = self.viewer.get_canvas()
@@ -334,7 +335,7 @@ class ViewerImageProxy:
         x, y, radius = int(x), int(y), int(radius)
         x1, x2 = x - radius, x + radius
         y1, y2 = y - radius, y + radius
-        data_np = self.cutout_data(x1, y1, x2, y2)
+        data_np = self.cutout_data(x1, y1, x2 + 1, y2 + 1)
 
         xarr = data_np[y - y1, :]
         yarr = data_np[:, x - x1]
@@ -379,9 +380,9 @@ class ViewerImageProxy:
             y1, y2 = max(xy_mn[1], y1), min(y2, xy_mx[1] - 1)
 
         # calculate pixel containment mask in bbox
-        yi = np.mgrid[y1:y2 + 1].reshape(-1, 1)
-        xi = np.mgrid[x1:x2 + 1].reshape(1, -1)
-        pts = np.asarray((xi, yi)).T
+        xi, yi = np.meshgrid(range(x1, x2 + 1), range(y1, y2 + 1))
+        pts = np.array((xi, yi)).T
+
         contains = shape_obj.contains_pts(pts)
 
         view = np.s_[y1:y2 + 1, x1:x2 + 1]
@@ -496,6 +497,17 @@ class ViewerImageProxy:
             return shape[-1]
         return 1
 
+    def get_minmax(self, noinf=False):
+        canvas = self.viewer.get_canvas()
+        cvs_imgs = list(self.get_images([], canvas))
+        if len(cvs_imgs) == 0:
+            return (0, 0)
+        mn, mx = cvs_imgs[0].get_image().get_minmax(noinf=noinf)
+        for cvs_img in cvs_imgs[1:]:
+            _mn, _mx = cvs_img.get_image().get_minmax(noinf=noinf)
+            mn, mx = min(mn, _mn), max(mx, _mx)
+        return (mn, mx)
+
     def get_shape(self):
         return self.shape
 
