@@ -121,7 +121,11 @@ class AstropyWCS(common.BaseWCS):
         skycrd = np.array([args], np.float_)
 
         try:
-            pix = self.wcs.wcs_world2pix(skycrd, origin)
+            pix = self.wcs.all_world2pix(skycrd, origin, maxiter=20,
+                                         detect_divergence=True, quiet=False)
+
+        except pywcs.NoConvergence as e:
+            pix = e.best_solution
 
         except Exception as e:
             self.logger.error("Error calculating radectopix: %s" % (str(e)))
@@ -150,7 +154,7 @@ class AstropyWCS(common.BaseWCS):
         # Skip in input and output is the same (no realize_frame
         # call in astropy)
         if to_class != frame_class:
-            coord = coord.transform_to(to_class)
+            coord = coord.transform_to(common.get_astropy_frame(to_class))
 
         return coord
 
@@ -192,7 +196,12 @@ class AstropyWCS(common.BaseWCS):
             if n > 0:
                 wcspt = np.hstack((wcspt, np.zeros((len(wcspt), n))))
         try:
-            datapt = self.wcs.all_world2pix(wcspt, origin)
+            datapt = self.wcs.all_world2pix(wcspt, origin, maxiter=20,
+                                            detect_divergence=True, quiet=False)
+
+        except pywcs.NoConvergence as e:
+            datapt = e.best_solution
+
         except Exception as e:
             self.logger.error(
                 "Error calculating wcspt_to_datapt: %s" % (str(e)))
@@ -243,7 +252,7 @@ class AstropyWCS(common.BaseWCS):
         # Skip if input and output is the same (no realize_frame
         # call in astropy)
         if to_class != frame_class:
-            coord = coord.transform_to(to_class)
+            coord = coord.transform_to(common.get_astropy_frame(to_class))
 
         return coord
 
