@@ -90,7 +90,7 @@ class ApplicationHandler(tornado.websocket.WebSocketHandler):
 
         # randomize the first timeout so we don't get every timer
         # expiring at the same time
-        interval = random.randint(1, self.interval)
+        interval = random.randint(1, self.interval)  # nosec
         delta = datetime.timedelta(milliseconds=interval)
         self.timeout = IOLoop.current().add_timeout(delta, self.timer_tick)
 
@@ -275,6 +275,16 @@ def get_icon(iconpath, size=None, format='png'):
     return icon
 
 
+def get_native_image(imgpath, format='png'):
+    print('image path', imgpath)
+    image = io_rgb.PILimage.open(imgpath)
+
+    img_buf = BytesIO()
+    image.save(img_buf, format=format)
+
+    return img_buf.getvalue()
+
+
 def font_info(font_str):
     """Extract font information from a font string, such as supplied to the
     'font' argument to a widget.
